;=======================================================================================
; Code : EasyTok librarie example
; Project Title : EasyTok librarie example
; Author : Philippe Agnisola
; Email :                    
; Version : 1.0.0
; Date :              
; Notes : 
;                   
;                   
;=======================================================================================

Include "Inc_TimeMachine.bb"
Include "Inc_EasyTOK.bb"

; ------------------------
; Freelook cam var
; ------------------------
Global Mouse_X_Speed#
Global Mouse_Y_Speed#
Global Camera_VelX#
Global Camera_VelZ#
Global Camera_Pitch
Global Camera_Yaw

; -------------------------------
; Refresh frame number
; -------------------------------
Global AppFPS =60

; ------------------
; Open screen
; ------------------
Graphics3D 1024, 768,32
SetBuffer BackBuffer ()

; ----------------------
; Change font text
; ----------------------
AppFont=LoadFont("Arial",15,1,0,0) 
SetFont AppFont

; -------------------------------------
; Setup camera and Easytok
; -------------------------------------
Global Cam=CreateCamera()
PositionEntity Cam,0,20,-40
TOK_InitWorld(0,-40,0,1000,100)

Proc_CreateVisor()

; -----------------------------
; Creating demo world
; -----------------------------
Global Light1=CreateLight(2)
PositionEntity Light1,-50,50,-50
LightColor Light1,250,150,50
LightRange Light1,100

Global Light2=CreateLight(2)
PositionEntity Light2,50,50,50
LightColor Light2,50,150,250
LightRange Light2,100

AmbientLight 100,90,80

Px#=0  : Py#=45 : Pz#=0
JointType=TOK_BallJoint%
Height=4

TOK_CreateGround("Floor",100,255,255,255)

; --------------------------
; Create the balloon
; --------------------------
Proc_CreateChain("Chaine01","./ETOK_Medias/chain.b3d","./ETOK_Medias/boulet.b3d",15,10,50,5,2)
Proc_CreateChain("Chaine02","./ETOK_Medias/chain.b3d","./ETOK_Medias/boulet.b3d",15,-10,50,5,2)
Proc_CreateChain("Chaine03","./ETOK_Medias/chain.b3d","./ETOK_Medias/boulet.b3d",15,0,50,5,2)

; -------------
; Main loop
; -------------
FramePeriod = 1000 / AppFPS
FrameTime = MilliSecs () - FramePeriod
MoveMouse GraphicsWidth()/2, GraphicsHeight()/2

Repeat
	; --------------------------
	; Frame rate limiting
	; --------------------------
	Repeat
		FrameElapsed = MilliSecs () - FrameTime
	Until FrameElapsed

	FrameTicks = FrameElapsed / FramePeriod
	FrameTween# = Float (FrameElapsed Mod FramePeriod) / Float (FramePeriod)

	; -----------------------
	; Update tweening
	; -----------------------
	For FrameLimit = 1 To FrameTicks
		If FrameLimit = FrameTicks Then
			CaptureWorld
		EndIf

		FrameTime = FrameTime + FramePeriod
		
		UpdateGame ()
		UpdateWorld
	Next

	; -------------------
	; Redraw world
	; -------------------
	TOK_RefreshEasyTOK()
	RenderWorld FrameTween
	
	; -----------------------
	; FPS Displaying
	; -----------------------
	Proc_CinemaStrip()
	Color 255,255,255
	Information$="<"+Get_FPS()+" FPS>"+" / <Obj Num : "+TOK_NumStructObject%+" / Joint Num : "+TOK_NumStructJoint%+"> / <Use Mouse Left to impulse>"
	Text GraphicsWidth()/2,GraphicsHeight()-30,Information$,True,True
	Flip
Until KeyHit (1)

; -----------
; End App
; -----------
TOK_FreePhysic()
FreeFont  AppFont
End

; -----------------------------------
; Tweening update function
; -----------------------------------
Function UpdateGame ()
	Proc_Freelook(Cam,1.05,0.05)
	
	; -------------------------------------------------
	; Apply a vertical force and movement
	; -------------------------------------------------
	If MouseDown(1) Then
		TOK_ApplyImpulse("Chaine0115",170,0,0)
	EndIf
	
	If MouseDown(2) Then 
		TOK_RefreshPhysic(0.3/AppFPS,10)
	Else
		TOK_RefreshPhysic(1.7/AppFPS,10)
	EndIf
End Function

; -----------------------
; Create the visor
; -----------------------
Function Proc_CreateVisor()
	Visor=LoadSprite("./ETOK_Medias/Target.png")
	PositionEntity Visor,EntityX(Cam),EntityY(Cam),EntityZ(Cam)+2
	ScaleSprite Visor,0.1,0.1
	EntityParent Visor,Cam
	EntityOrder Visor,-1
End Function

; ----------------------------------
; Create the cinema strip
; ----------------------------------
Function Proc_CinemaStrip(Size=60)
	Color 1,1,1
	Rect 0,0,GraphicsWidth(),Size
	Rect 0,GraphicsHeight()-Size,GraphicsWidth(),Size
End Function

; ----------------------
; Camera freelook
; ----------------------
Function Proc_Freelook(CamEntity,Velocity#,Speed#)
	Mouse_X_Speed#=MouseXSpeed()*0.5
	Mouse_Y_Speed#=MouseYSpeed()*0.5

	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	Camera_Pitch=Camera_Pitch+Mouse_Y_Speed#
	Camera_Yaw=Camera_Yaw+Mouse_X_Speed#

	RotateEntity CamEntity,Camera_Pitch,-Camera_Yaw,0

	If KeyDown(203) Then
		Camera_VelX=Camera_VelX-Speed#
	ElseIf  KeyDown(205)
		Camera_VelX=Camera_VelX+Speed#
	EndIf
	
	If KeyDown(208) Then
		Camera_VelZ=Camera_VelZ-Speed#
	ElseIf  KeyDown(200)
		Camera_VelZ=Camera_VelZ+Speed#
	EndIf
 
	Camera_VelX=Camera_VelX/velocity#
	Camera_VelZ=Camera_VelZ/velocity#
	MoveEntity CamEntity,Camera_VelX,0,Camera_VelZ

	If KeyDown(200) Then
		MoveEntity CamEntity,0,0,Speed#
	EndIf

	If KeyDown(208) Then
		MoveEntity CamEntity,0,0,-Speed#
	EndIf

	If KeyDown(205) Then
		MoveEntity CamEntity,Speed#,0,0
	EndIf

	If KeyDown(203) Then
		MoveEntity CamEntity,-Speed#,0,0
	EndIf
End Function

; -------------------------------------------------------------
; Little function to create a chain with mesh
; -------------------------------------------------------------
Function Proc_CreateChain(Name$,ChainFile$,BallFile$,Number,Px#,Py#,Pz#,Mass#,View%=0)
	SizeHeight#=1.2
	Tmp_ID=1

	Tmp_Chain=LoadMesh(ChainFile$)
	Tmp_Ball=LoadMesh(BallFile$)
	

	For i=1 To Number
		Py#=Py#-(SizeHeight*2)

		Yaw#=I*65

		If i<Number Then
			MyCube=TOK_CreateRBCube(Name$+I,Tmp_Chain,Px#,Py#-SizeHeight#,Pz#,0,Yaw#,0,0.97,0.97,0.97,Mass#,"",255,255,255,1,0)
		Else
			SizeHeight#=3.1
			MyCube=TOK_CreateRBSphere(Name$+i,Tmp_Ball,Px#,Py#-SizeHeight#,Pz#,0,Yaw#,0,1,Mass#*2,"",255,255,255,1,0)

			DebugLog Name$+i
		EndIf
		
		TOK_CreateJoint(Name$+Joint+I,Tmp_ID,Name$+I,MyCubeOld$,Px#,Py#,Pz#,0,0,0,View%,TOK_BallJoint%,100,-60,60)
		MyCubeOld$=Name$+I : Tmp_ID=2
	Next

	FreeEntity Tmp_chain
	FreeEntity Tmp_Ball
End Function 