;=======================================================================================
; Code :
; Project Title :
; Author : Philippe Agnisola
; Email :                    
; Version : 1.0.0
; Date :              
; Notes : 
;                   
;                   
;=======================================================================================

Include "Inc_TimeMachine.bb"
Include "Inc_EasyTOK.bb"

; ------------------------
; Freelook cam var
; ------------------------
Global Mouse_X_Speed#
Global Mouse_Y_Speed#
Global Camera_VelX#
Global Camera_VelZ#
Global Camera_Pitch
Global Camera_Yaw

; ------------------------------
; Nombre d'image MAX
; ------------------------------
Global AppFPS =60

; ---------------------------
; Ouverture de l'ecran
; ---------------------------
Graphics3D 1024, 768,32,0
SetBuffer BackBuffer ()

; ---------------------------------
; Changement de la fonte
; ---------------------------------
AppFont=LoadFont("Arial",15,1,0,0) 
SetFont AppFont

; ----------------------------------------
; Setup camera and Easytok
; ----------------------------------------
Global Cam=CreateCamera()
PositionEntity Cam,10,40,-40

Proc_CreateVisor()

TOK_InitWorld(0,-40,0,1000,100)
TOK_CreateGround("Floor",150,255,200,100)

; -----------------------------
; Creating demo world
; -----------------------------
Global Light1=CreateLight(2)
PositionEntity Light1,-50,50,-50
LightColor Light1,250,150,50
LightRange Light1,100

Global Light2=CreateLight(2)
PositionEntity Light2,50,50,50
LightColor Light2,50,150,250
LightRange Light2,100

AmbientLight 80,90,100

TOK_CreateWallScene()

; -----------------------------------------------
;	Boucle principale du programme
; -----------------------------------------------
FramePeriod = 1000 / AppFPS
FrameTime = MilliSecs () - FramePeriod
MoveMouse GraphicsWidth()/2, GraphicsHeight()/2

Repeat
	; --------------------------
	; Frame rate limiting
	; --------------------------
	Repeat
		FrameElapsed = MilliSecs () - FrameTime
	Until FrameElapsed

	FrameTicks = FrameElapsed / FramePeriod
	FrameTween# = Float (FrameElapsed Mod FramePeriod) / Float (FramePeriod)

	; -----------------------
	; Update tweening
	; -----------------------
	For FrameLimit = 1 To FrameTicks
		If FrameLimit = FrameTicks Then
			CaptureWorld
		EndIf

		FrameTime = FrameTime + FramePeriod
		
		UpdateGame ()
		UpdateWorld
	Next

	
	; -------------------
	; Redraw world
	; -------------------
	TOK_RefreshEasyTOK()
	RenderWorld FrameTween
	
	; ---------------------
	; FPS Displaying
	; ---------------------
	Proc_CinemaStrip()
	Color 255,255,255
	Information$="<"+Get_FPS()+" FPS> / "+"<Arrow key to move> <Left button = Launch bomb> <Wheel button = Recreate wall> <Right button = Activate all bombs>"
	Text GraphicsWidth()/2,GraphicsHeight()-30,Information$,True,True
	Flip
Until KeyHit (1)

; -------------
; End App
; -------------
TOK_FreePhysic()
FreeFont  AppFont
End

; -------------------------------------
; Tweening update function
; -------------------------------------
Function UpdateGame ()
	Proc_Freelook(Cam,1.05,0.05)

	Range=200
	Force=90
	
	If MouseHit(1) Then
		TOK_CreateBomb("Bomb"+Rand(100,255),EntityX#(Cam),EntityY#(Cam),EntityZ#(Cam),Range,Force,1000)
	EndIf

	If MouseHit(3) Then
		TOK_CreateWallScene()
	EndIf

	If MouseHit(2)
		TOK_ActivateAllBomb()
	EndIf

	; ---------------------	
	; Update physic
	; ---------------------
	TOK_RefreshPhysic(0.2/AppFPS,10)
End Function

; ----------------------
; Camera freelook
; ----------------------
Function Proc_Freelook(CamEntity,Velocity#,Speed#)
 Mouse_X_Speed#=MouseXSpeed()*0.5
 Mouse_Y_Speed#=MouseYSpeed()*0.5

 MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
 Camera_Pitch=Camera_Pitch+Mouse_Y_Speed#
 Camera_Yaw=Camera_Yaw+Mouse_X_Speed#


 RotateEntity CamEntity,Camera_Pitch,-Camera_Yaw,0

 If KeyDown(203) Then
   Camera_VelX=Camera_VelX-Speed#
 ElseIf  KeyDown(205)
   Camera_VelX=Camera_VelX+Speed#
 EndIf
 If KeyDown(208) Then
   Camera_VelZ=Camera_VelZ-Speed#
 ElseIf  KeyDown(200)
   Camera_VelZ=Camera_VelZ+Speed#
 EndIf
 
 Camera_VelX=Camera_VelX/velocity#
 Camera_VelZ=Camera_VelZ/velocity#
 MoveEntity CamEntity,Camera_VelX,0,Camera_VelZ

 If KeyDown(200) Then
   MoveEntity CamEntity,0,0,Speed#
 EndIf

 If KeyDown(208) Then
   MoveEntity CamEntity,0,0,-Speed#
 EndIf

 If KeyDown(205) Then
   MoveEntity CamEntity,Speed#,0,0
 EndIf

 If KeyDown(203) Then
   MoveEntity CamEntity,-Speed#,0,0
 EndIf
End Function

; -----------------------
; Create the visor
; -----------------------
Function Proc_CreateVisor()
	Visor=LoadSprite("./ETOK_Medias/Target.png")
	PositionEntity Visor,EntityX(Cam),EntityY(Cam),EntityZ(Cam)+2
	ScaleSprite Visor,0.1,0.1
	EntityParent Visor,Cam
	EntityOrder Visor,-1
End Function

; ----------------------------------
; Create the cinema strip
; ----------------------------------
Function Proc_CinemaStrip(Size=60)
	Color 1,1,1
	Rect 0,0,GraphicsWidth(),Size
	Rect 0,GraphicsHeight()-Size,GraphicsWidth(),Size
End Function


; ----------------------------------------------------
; Functions to create easily wall brick
; ----------------------------------------------------
Const WallCube_Scale=3.9

Function TOK_CreateWallScene()
	TOK_DeleteWallCube()

	How_Many=16
	
	Wall_Area=15
	Wall_Height#=4.5

	For I=1 To How_Many
		Decal=1-Decal
		
		TOK_CreateWallCube(Decal*8.5,Wall_Height#,0,Wall_Area)
		Wall_Height#=Wall_Height#+8.1
	Next
End Function

Function TOK_CreateWallCube(Px#,Py#,Pz#,Number)
	For I=-Number To  Number Step WallCube_Scale*2
		TOK_CreateRBCube("Wall_"+I,False,Px#+(I*2),Py#,Pz#,0,0,0,(WallCube_Scale*2)-0.1,WallCube_Scale,WallCube_Scale,50,"",Rnd(255),Rnd(255),Rnd(255),1,0)

		TOK_SetRBSleepingParameter("Wall_"+I,0.1)	
	Next
End Function

Function TOK_DeleteWallCube()
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		TOK_DeleteRBEntity(G\Name$)
	Next 
End Function