;=======================================================================================
; Code : EasyTok librarie example
; Project Title : EasyTok librarie example
; Author : Philippe Agnisola
; Email :                    
; Version : 1.0.0
; Date :              
; Notes : 
;                   
;                   
;=======================================================================================

Include "Inc_TimeMachine.bb"
Include "Inc_EasyTOK.bb"

; ------------------------
; Freelook cam var
; ------------------------
Global Mouse_X_Speed#
Global Mouse_Y_Speed#
Global Camera_VelX#
Global Camera_VelZ#
Global Camera_Pitch
Global Camera_Yaw

; -------------------------------
; Refresh frame number
; -------------------------------
Global AppFPS =60

; ------------------
; Open screen
; ------------------
Graphics3D 1024, 768,32,0
SetBuffer BackBuffer ()

; ----------------------
; Change font text
; ----------------------
AppFont=LoadFont("Arial",15,1,0,0) 
SetFont AppFont

; -------------------------------------
; Setup camera and Easytok
; -------------------------------------
Global Cam=CreateCamera()
PositionEntity Cam,0,20,-40

Proc_CreateVisor()

TOK_InitWorld(0,-40,0,1000,100)
TOK_CreateGround("Floor",150,255,150,10)

; -----------------------------
; Creating demo world
; -----------------------------
Global Light1=CreateLight(2)
PositionEntity Light1,-50,50,-50
LightColor Light1,250,150,50
LightRange Light1,100

Global Light2=CreateLight(2)
PositionEntity Light2,50,50,50
LightColor Light2,50,150,250
LightRange Light2,100

AmbientLight 80,90,100

; ---------------------------
; Ball and the cubes
; ---------------------------
Py#=12

Global TheMan00=TOK_CreateRBCube("Toto0",False,-5,Py#,0,0,0,0,6,6,6,0.5,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto0",1=10,50,250)
TOK_SetRBLinearDamping("Toto0",0.03) ; Try 0.1 to damp the blue cube


Global Chain01=TOK_CreateRBSphere("Toto1",False,10,Py#,0,0,0,0,2,1,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto1",110,0,150)

Global Chain02=TOK_CreateRBSphere("Toto2",False,20,Py#,0,0,0,0,2,1,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto2",110,150,110)

Global Chain03=TOK_CreateRBSphere("Toto3",False,30,Py#,0,0,0,0,2,1,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto3",110,150,110)

Global Chain04=TOK_CreateRBSphere("Toto4",False,40,Py#,0,0,0,0,2,1,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto4",110,150,110)

Global TheDog=TOK_CreateRBSphere("Toto5",False,50,Py#,0,0,0,0,6,2,"",255,Rnd(255),255,1,-1)
TOK_EntityColor("Toto5",0,150,0)


TOK_CreateJoint("MyJoint00",2,"Toto0","Toto1",5,Py#,0,90,0,0,1,TOK_BallJoint%,100,-180,180)
TOK_CreateJoint("MyJoint01",2,"Toto1","Toto2",15,Py#,0,90,0,0,1,TOK_BallJoint%,100,-180,180)
TOK_CreateJoint("MyJoint02",2,"Toto2","Toto3",25,Py#,0,90,0,0,1,TOK_BallJoint%,100,-180,180)
TOK_CreateJoint("MyJoint03",2,"Toto3","Toto4",35,Py#,0,90,0,0,1,TOK_BallJoint%,100,-180,180)
TOK_CreateJoint("MyJoint04",2,"Toto4","Toto5",45,Py#,0,90,0,0,1,TOK_BallJoint%,100,-180,180)


Global Angle#

; -------------
; Main loop
; -------------
FramePeriod = 1000 / AppFPS
FrameTime = MilliSecs () - FramePeriod
MoveMouse GraphicsWidth()/2, GraphicsHeight()/2

Repeat
	; --------------------------
	; Frame rate limiting
	; --------------------------
	Repeat
		FrameElapsed = MilliSecs () - FrameTime
	Until FrameElapsed

	FrameTicks = FrameElapsed / FramePeriod
	FrameTween# = Float (FrameElapsed Mod FramePeriod) / Float (FramePeriod)

	; -----------------------
	; Update tweening
	; -----------------------
	For FrameLimit = 1 To FrameTicks
		If FrameLimit = FrameTicks Then
			CaptureWorld
		EndIf

		FrameTime = FrameTime + FramePeriod
		
		UpdateGame ()
		UpdateWorld
	Next

	; -------------------
	; Redraw world
	; -------------------
	TOK_RefreshEasyTOK()
	RenderWorld FrameTween
	
	; -----------------------
	; FPS Displaying
	; -----------------------
	Proc_CinemaStrip()
	Color 255,255,255
	Information$="<"+Get_FPS()+" FPS> / "+"<Arrow key to move> / <Mouse left to launch objects> / <Numerical arrow key to move green ball>"
	Text GraphicsWidth()/2,GraphicsHeight()-30,Information$,True,True
	Flip
Until KeyHit (1)

; -----------
; End App
; -----------
TOK_FreePhysic()
FreeFont  AppFont
End

; -----------------------------------
; Tweening update function
; -----------------------------------
Function UpdateGame ()
	Proc_Freelook(Cam,1.05,0.05)

	; -----------------------------------------
	; Launch a cube and a sphere
	; -----------------------------------------
	Force=20
	
	If KeyDown(75) Then 
		TOK_ApplyImpulse("Toto5",-Force,0,0)
	EndIf

	If KeyDown(77) Then 
		TOK_ApplyImpulse("Toto5",Force,0,0)
	EndIf

	If KeyDown(72) Then 
		TOK_ApplyImpulse("Toto5",0,0,Force)
	EndIf

	If KeyDown(80) Then 
		TOK_ApplyImpulse("Toto5",0,0,-Force)
	EndIf

	
	TOK_RefreshPhysic(1.7/AppFPS,30)
End Function

; -----------------------
; Create the visor
; -----------------------
Function Proc_CreateVisor()
	Visor=LoadSprite("./ETOK_Medias/Target.png")
	PositionEntity Visor,EntityX(Cam),EntityY(Cam),EntityZ(Cam)+2
	ScaleSprite Visor,0.1,0.1
	EntityParent Visor,Cam
	EntityOrder Visor,-1
End Function

; ----------------------------------
; Create the cinema strip
; ----------------------------------
Function Proc_CinemaStrip(Size=60)
	Color 1,1,1
	Rect 0,0,GraphicsWidth(),Size
	Rect 0,GraphicsHeight()-Size,GraphicsWidth(),Size
End Function

; ----------------------
; Camera freelook
; ----------------------
Function Proc_Freelook(CamEntity,Velocity#,Speed#)
	Mouse_X_Speed#=MouseXSpeed()*0.5
	Mouse_Y_Speed#=MouseYSpeed()*0.5

	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	Camera_Pitch=Camera_Pitch+Mouse_Y_Speed#
	Camera_Yaw=Camera_Yaw+Mouse_X_Speed#

	RotateEntity CamEntity,Camera_Pitch,-Camera_Yaw,0

	If KeyDown(203) Then
		Camera_VelX=Camera_VelX-Speed#
	ElseIf  KeyDown(205)
		Camera_VelX=Camera_VelX+Speed#
	EndIf
	
	If KeyDown(208) Then
		Camera_VelZ=Camera_VelZ-Speed#
	ElseIf  KeyDown(200)
		Camera_VelZ=Camera_VelZ+Speed#
	EndIf
 
	Camera_VelX=Camera_VelX/velocity#
	Camera_VelZ=Camera_VelZ/velocity#
	MoveEntity CamEntity,Camera_VelX,0,Camera_VelZ

	If KeyDown(200) Then
		MoveEntity CamEntity,0,0,Speed#
	EndIf

	If KeyDown(208) Then
		MoveEntity CamEntity,0,0,-Speed#
	EndIf

	If KeyDown(205) Then
		MoveEntity CamEntity,Speed#,0,0
	EndIf

	If KeyDown(203) Then
		MoveEntity CamEntity,-Speed#,0,0
	EndIf
End Function