;=======================================================================================
; Code : EasyTok librarie example
; Project Title : EasyTok librarie example
; Author : Philippe Agnisola
; Email :                    
; Version : 1.0.0
; Date :              
; Notes : 
;                   
;                   
;=======================================================================================

Include "Inc_TimeMachine.bb"
Include "Inc_EasyTOK.bb"

; ------------------------
; Freelook cam var
; ------------------------
Global Mouse_X_Speed#
Global Mouse_Y_Speed#
Global Camera_VelX#
Global Camera_VelZ#
Global Camera_Pitch
Global Camera_Yaw

; -------------------------------
; Refresh frame number
; -------------------------------
Global AppFPS =60

; ------------------
; Open screen
; ------------------
Graphics3D 1024, 768,32
SetBuffer BackBuffer ()

; ----------------------
; Change font text
; ----------------------
AppFont=LoadFont("Arial",15,1,0,0) 
SetFont AppFont

; ---------------------------------------
; Setup camera and Easytok
; ---------------------------------------
Global Cam=CreateCamera()
PositionEntity Cam,0,20,-40
EntityRadius Cam,3
EntityType Cam,1

Proc_CreateVisor()

TOK_InitWorld(0,-40,0,1000,100)

; -----------------------------
; Creating demo world
; -----------------------------
Global Light1=CreateLight(2)
PositionEntity Light1,-50,50,-50
LightColor Light1,200,100,50
LightRange Light1,60

Global Light2=CreateLight(2)
PositionEntity Light2,50,50,50
LightColor Light2,50,200,50
LightRange Light2,60

AmbientLight 50,50,60

; ----------------------------------------------------
; Load the tokamak world from a B3D
; ----------------------------------------------------
TOK_CreatePhysicFromMesh(LoadAnimMesh("./ETOK_Medias/Room02.b3d"))
Collisions 1,2,2,3

Global WaterPlan=CreatePlane(5)
PositionEntity WaterPlan,0,-3,0
EntityFX WaterPlan,1+16
EntityBlend WaterPlan,3
EntityShininess WaterPlan,1

Global WaterTexture01=LoadTexture("ETOK_Medias/Water01.jpg")
EntityTexture WaterPlan,WaterTexture01,0,1
ScaleTexture WaterTexture01,50,50

Global WaterTexture02=LoadTexture("ETOK_Medias/Water01.jpg")
EntityTexture WaterPlan,WaterTexture02,0,2
ScaleTexture WaterTexture02,60,60
EntityAlpha WaterPlan,0.6

; -------------
; Main loop
; -------------
FramePeriod = 1000 / AppFPS
FrameTime = MilliSecs () - FramePeriod
MoveMouse GraphicsWidth()/2, GraphicsHeight()/2

Global MyShootRate=250
Global MyShootTimer=TOK_CreateTimer( MyShootRate)

Repeat
	; --------------------------
	; Frame rate limiting
	; --------------------------
	Repeat
		FrameElapsed = MilliSecs () - FrameTime
	Until FrameElapsed

	FrameTicks = FrameElapsed / FramePeriod
	FrameTween# = Float (FrameElapsed Mod FramePeriod) / Float (FramePeriod)

	; -----------------------
	; Update tweening
	; -----------------------
	For FrameLimit = 1 To FrameTicks
		If FrameLimit = FrameTicks Then
			CaptureWorld
		EndIf

		FrameTime = FrameTime + FramePeriod
		
		UpdateGame ()
		UpdateWorld
	Next

	; -------------------
	; Redraw world
	; -------------------
	TOK_RefreshEasyTOK()
	RenderWorld FrameTween
	
	; -----------------------
	; FPS Displaying
	; -----------------------
	Proc_CinemaStrip()
	Color 255,255,255
	Information$="<"+Get_FPS()+" FPS> / "+"<Arrow key to move> / <Mouse left to shoot> / <Use camera to push objects>"
	Text GraphicsWidth()/2,GraphicsHeight()-30,Information$,True,True
	Flip
Until KeyHit (1)

; -----------
; End App
; -----------
TOK_FreePhysic()
FreeFont  AppFont
End

; -----------------------------------
; Tweening update function
; -----------------------------------
Function UpdateGame ()
	Proc_Freelook(Cam,1.05,0.05)

	; --------------------------------------
	; Shoot objects with mouse
	; --------------------------------------
	If MouseDown(1) Then
		If  TOK_TimeOut(MyShootTimer) Then
		
			Tmp$="Bullet"+Rnd(1000)
			TOK_CreateRBSphere(Tmp$,False,EntityX(Cam),EntityY(Cam),EntityZ(Cam),EntityPitch(Cam),EntityYaw(cam),EntityRoll(cam),2,6,"",Rnd(255),Rnd(255),Rnd(255),1,3)
			TOK_ApplyImpulseWithAngle(Tmp$,EntityPitch(Cam),EntityYaw(cam),300)
			TOK_ApplyMaterial(Tmp$,TOK_TypeRigidBody%,"Metal")
	
			MyShootTimer=TOK_CreateTimer( MyShootRate)
		EndIf
	EndIf
	
	; ------------------------------
	; Moving water texture
	; ------------------------------
	PositionTexture WaterTexture01,-Sin(MilliSecs()/36)/10,-Cos(MilliSecs()/36)/10
	PositionTexture WaterTexture02,Sin(MilliSecs()/36)/10,Cos(MilliSecs()/36)/10
	PositionEntity WaterPlan,0,-3+Sin(MilliSecs()/10),0

	; ----------------------------------------
	; Moving the box in the water
	; ----------------------------------------
	Proc_CreateWaterSwing(-4,Tmp$,False,True,False)
	Proc_CreateWaterSwing(-4,"CASE01",True,False,True)
	Proc_CreateWaterSwing(-4,"CASE02",False,True,False)
	Proc_CreateWaterSwing(-4,"CASE03",False,True,True)

	Proc_CreateWaterSwing(-4,"CASE04",True,False,True)
	Proc_CreateWaterSwing(-4,"CASE05",False,True,False)
	Proc_CreateWaterSwing(-4,"CASE06",False,True,True)
	
	; -----------------------------------------------------------
	; Define if the camera collide a toka object
	; -----------------------------------------------------------
	Local Power=100
	
	If CountCollisions(Cam)>0 Then
		For k=1 To CountCollisions(cam)
			Entity_Collided=CollisionEntity(Cam,k)

			If TOK_IsRigidBody(Entity_Collided)=True Then
				TOK_ApplyImpulse2(TOK_GetRBName$(Entity_Collided),-Power*CollisionNX(Cam,k),-Power*CollisionNY(Cam,k),-Power*CollisionNZ(Cam,k),CollisionX(Cam,k),CollisionY(Cam,k),CollisionZ(Cam,k))
			EndIf
		Next
	EndIf

	; ---------------------	
	; Update physic
	; ---------------------
	TOK_RefreshPhysic(1.7/AppFPS,5)
End Function

; -----------------------
; Create the visor
; -----------------------
Function Proc_CreateVisor()
	Visor=LoadSprite("./ETOK_Medias/Target.png")
	PositionEntity Visor,EntityX(Cam),EntityY(Cam),EntityZ(Cam)+2
	ScaleSprite Visor,0.1,0.1
	EntityParent Visor,Cam
	EntityOrder Visor,-1
End Function

; ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; Create the water float this function is dependent that the objects mass and the world gravity ! adjust with your hown setting
; ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Function Proc_CreateWaterSwing(WaterHeight#,Name$,AdjustX=True,AdjustY=True,AdjustZ=True)
	; -------------------------------
	; Get object Y position
	; -------------------------------
 	InWater#=TOK_GetRBY#(Name$)

	; -------------------------------------
	; If objects are under water
	; -------------------------------------
	If InWater#<WaterHeight# Then
		TOK_SetRBMass(Name$,1.2)
		TOK_SetRBLinearDamping(Name$,0.01)
		TOK_SetRBAngularDamping(Name$,0.01)

		Vol#=(1.0/30.0)*Abs(InWater#*2.3)

		If AdjustX=True Then 
			ForceX#=-1
		Else
			ForceX#=1
		EndIf

		If AdjustZ=True Then
			ForceZ#=-1
		Else
			ForceZ#=1
		EndIf

		If AdjustY=True Then
			TwistY#=-Vol#*60
			TwistZ#=Cos(MilliSecs()/10)*700
		Else
			TwistY#=Vol#*60
			TwistZ#=-Sin(MilliSecs()/10)*800
		EndIf
		
		TOK_ApplyForce(Name$,ForceX#,70*vol,ForceZ#)
		TOK_ApplyTwist(Name$,TwistZ#,TwistY#,TwistZ#)
	EndIf

	; ---------------------------------------
	; If objects are outside water
	; ---------------------------------------
	If InWater#>1
		TOK_ApplyForce(Name$,0,0,0)
		TOK_SetRBMass(Name$,30)
		TOK_SetRBLinearDamping(Name$,0.0001)
		TOK_SetRBAngularDamping(Name$,0.0001)
	EndIf	
End Function

; ----------------------------------
; Create the cinema strip
; ----------------------------------
Function Proc_CinemaStrip(Size=60)
	Color 1,1,1
	Rect 0,0,GraphicsWidth(),Size
	Rect 0,GraphicsHeight()-Size,GraphicsWidth(),Size
End Function

; ----------------------
; Camera freelook
; ----------------------
Function Proc_Freelook(CamEntity,Velocity#,Speed#)
	Mouse_X_Speed#=MouseXSpeed()*0.5
	Mouse_Y_Speed#=MouseYSpeed()*0.5

	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	Camera_Pitch=Camera_Pitch+Mouse_Y_Speed#
	Camera_Yaw=Camera_Yaw+Mouse_X_Speed#

	RotateEntity CamEntity,Camera_Pitch,-Camera_Yaw,0

	If KeyDown(203) Then
		Camera_VelX=Camera_VelX-Speed#
	ElseIf  KeyDown(205)
		Camera_VelX=Camera_VelX+Speed#
	EndIf
	
	If KeyDown(208) Then
		Camera_VelZ=Camera_VelZ-Speed#
	ElseIf  KeyDown(200)
		Camera_VelZ=Camera_VelZ+Speed#
	EndIf
 
	Camera_VelX=Camera_VelX/velocity#
	Camera_VelZ=Camera_VelZ/velocity#
	MoveEntity CamEntity,Camera_VelX,0,Camera_VelZ

	If KeyDown(200) Then
		MoveEntity CamEntity,0,0,Speed#
	EndIf

	If KeyDown(208) Then
		MoveEntity CamEntity,0,0,-Speed#
	EndIf

	If KeyDown(205) Then
		MoveEntity CamEntity,Speed#,0,0
	EndIf

	If KeyDown(203) Then
		MoveEntity CamEntity,-Speed#,0,0
	EndIf
End Function