;=======================================================================================
; Code : EasyTok librarie example
; Project Title : EasyTok librarie example
; Author : Philippe Agnisola
; Email :                    
; Version : 1.0.0
; Date :              
; Notes : 
;                   
;                   
;=======================================================================================

Include "Inc_TimeMachine.bb"
Include "Inc_EasyTOK.bb"

; ------------------------
; Freelook cam var
; ------------------------
Global Mouse_X_Speed#
Global Mouse_Y_Speed#
Global Camera_VelX#
Global Camera_VelZ#
Global Camera_Pitch
Global Camera_Yaw

; -------------------------------
; Refresh frame number
; -------------------------------
Global AppFPS =60

; ------------------
; Open screen
; ------------------
Graphics3D 1024, 768,32
SetBuffer BackBuffer ()

; ----------------------
; Change font text
; ----------------------
AppFont=LoadFont("Arial",15,1,0,0) 
SetFont AppFont

; -------------------------------------
; Setup camera and Easytok
; -------------------------------------
Global Cam=CreateCamera()
PositionEntity Cam,0,30,-50

Proc_CreateVisor()

TOK_InitWorld(0,-41,0,1000,100)
TOK_CreateGround("Floor",100,255,150,10)

; -----------------------------
; Creating demo world
; -----------------------------
Global Light1=CreateLight(2)
PositionEntity Light1,-50,50,-50
LightColor Light1,250,150,50
LightRange Light1,100

Global Light2=CreateLight(2)
PositionEntity Light2,50,50,50
LightColor Light2,50,150,250
LightRange Light2,100

AmbientLight 80,90,100

Cube_Scale=4
TOK_CreateRBCylinder("Cylinder",False,0,24,60,90,90,0,Cube_Scale,Cube_Scale*10,20,"",Rnd(255),Rnd(255),Rnd(255),1,0)
TOK_ApplyMaterial("Cylinder",TOK_TypeRigidBody%,"Ice")

; --------------------------------
; Define the ragdoll part
; --------------------------------
Type TOK_Ragdoll
	Field Ragdoll_Name$
	Field Ragdoll_Mass#
	
	Field Ragdoll_Px#
	Field Ragdoll_Py#
	Field Ragdoll_Pz#
	
	Field Part_Head
	Field Part_Torso
	Field Part_Pelvis
	
	Field Part_Left_Arm_01
	Field Part_Right_Arm_01

	Field Part_Left_Arm_02
	Field Part_Right_Arm_02

	Field Part_Left_Leg_01
	Field Part_Right_Leg_01

	Field Part_Left_Leg_02
	Field Part_Right_Leg_02
End Type

For i=1 To 4
	TOK_CreateRagdoll("Doll0"+I,Rnd(-35,35),25,Rnd(-20,20))
Next 

; ------------------------------------------------------
; This is an example of ragdoll structur
; ------------------------------------------------------
Function TOK_CreateRagdoll(Name$,Px#,Py#,Pz#)
		R.TOK_Ragdoll = New TOK_Ragdoll
		R\Ragdoll_Name$=Name$
		R\Ragdoll_Mass#=1

		R\Ragdoll_Px#=Px#
		R\Ragdoll_Py#=Py#
		R\Ragdoll_Pz#=Pz#

		R\Part_Head=TOK_CreateRBSphere(Name$+"_"+"Head",False,Px#,Py#,Pz#,0,0,0,2,R\Ragdoll_Mass#,"",255,255,255,1,0)
		R\Part_Torso=TOK_CreateRBCube(Name$+"_"+"Torso",False,Px#,Py#-5,Pz#,0,0,0,2,3,1,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint01",2,Name$+"_"+"Head",Name$+"_"+"Torso",Px#,Py#-2,Pz#,0,0,90,1,TOK_HingeJoint%,100,-45,45)
		
		R\Part_Pelvis=TOK_CreateRBCube(Name$+"_"+"Pelvis",False,Px#,Py#-10,Pz#,0,0,0,2,1.5,1,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint02",2,Name$+"_"+"Torso",Name$+"_"+"Pelvis",Px#,Py#-8,Pz#,0,0,90,1,TOK_HingeJoint%,100,-30,60)
	
		R\Part_Left_Arm_01=TOK_CreateRBCube(Name$+"_"+"LeftArm1",False,Px#-5,Py#-4.5,Pz#,0,0,-45,0.5,2.5,0.5,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint03",2,Name$+"_"+"Torso",Name$+"_"+"LeftArm1",Px#-2.5,Py#-2.5,Pz#,90,0,0,1,TOK_BallJoint%,100,-60,40)
		
		R\Part_Right_Arm_01=TOK_CreateRBCube(Name$+"_"+"RightArm1",False,Px#+5,Py#-4.5,Pz#,0,0,45,0.5,2.5,0.5,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint05",2,Name$+"_"+"Torso",Name$+"_"+"RightArm1",Px#+2.5,Py#-2.5,Pz#,90,0,0,1,TOK_BallJoint%,100,-60,40)

		R\Part_Left_Arm_02=TOK_CreateRBCube(Name$+"_"+"LeftArm2",False,Px#-9,Py#-8.5,Pz#,0,0,-45,0.5,2.5,0.5,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint04",2,Name$+"_"+"LeftArm1",Name$+"_"+"LeftArm2",Px#-7,Py#-6.5,Pz#,0,0,0,1,TOK_HingeJoint%,100,-60,40)
		
		R\Part_Right_Arm_02=TOK_CreateRBCube(Name$+"_"+"RightArm2",False,Px#+9,Py#-8.5,Pz#,0,0,45,0.5,2.5,0.5,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint06",2,Name$+"_"+"RightArm1",Name$+"_"+"RightArm2",Px#+7,Py#-6.5,Pz#,0,0,0,1,TOK_HingeJoint%,100,-60,40)

		R\Part_Left_Leg_01=TOK_CreateRBCube(Name$+"_"+"LeftLeg1",False,Px#-2,Py#-15,Pz#,0,0,0,0.7,2.5,0.7,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint07",2,Name$+"_"+"LeftLeg1",Name$+"_"+"Pelvis",Px#-2,Py#-12,Pz#,0,0,90,1,TOK_HingeJoint%,100,-45,90)
		
		R\Part_Right_Leg_01=TOK_CreateRBCube(Name$+"_"+"RightLeg1",False,Px#+2,Py#-15,Pz#,0,0,0,0.7,2.5,0.7,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint08",2,Name$+"_"+"RightLeg1",Name$+"_"+"Pelvis",Px#+2,Py#-12,Pz#,0,0,90,1,TOK_HingeJoint%,100,-45,90)

		R\Part_Left_Leg_02=TOK_CreateRBCube(Name$+"_"+"LeftLeg2",False,Px#-2,Py#-21,Pz#,0,0,0,0.7,2.5,0.7,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint09",2,Name$+"_"+"LeftLeg2",Name$+"_"+"LeftLeg1",Px#-2,Py#-18,Pz#,0,0,90,1,TOK_HingeJoint%,100,0,150)
		
		R\Part_Right_Leg_02=TOK_CreateRBCube(Name$+"_"+"RightLeg2",False,Px#+2,Py#-21,Pz#,0,0,0,0.7,2.5,0.7,R\Ragdoll_Mass#,"",255,255,255,1,0)
		TOK_CreateJoint(Name$+"_"+"Joint10",2,Name$+"_"+"RightLeg2",Name$+"_"+"RightLeg1",Px#+2,Py#-18,Pz#,0,0,90,1,TOK_HingeJoint%,100,0,150)

		TOK_ApplyForce(Name$+"_"+"Head",0,330,0)
End Function

Function TOK_ResetRagdoll()
	For R.TOK_Ragdoll = Each TOK_Ragdoll
		TOK_ApplyForce(R\Ragdoll_Name$+"_"+"Head",0,1950,0)
	Next 
End Function

Function TOK_DeadRagdoll()
	For R.TOK_Ragdoll = Each TOK_Ragdoll
		TOK_ApplyForce(R\Ragdoll_Name$+"_"+"Head",0,0,0)
	Next 
End Function

Function TOK_KillRagdoll()
	For R.TOK_Ragdoll = Each TOK_Ragdoll
		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"Head")
		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"Torso")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint01")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"Pelvis")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint02")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"LeftArm1")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint03")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"RightArm1")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint05")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"LeftArm2")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint04")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"RightArm2")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint06")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"LeftLeg1")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint07")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"RightLeg1")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint08")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"LeftLeg2")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint09")

		TOK_DeleteRBEntity(R\Ragdoll_Name$+"_"+"RightLeg2")
		TOK_DeleteJoint(R\Ragdoll_Name$+"_"+"Joint10")
	Next
End Function	

; -------------
; Main loop
; -------------
FramePeriod = 1000 / AppFPS
FrameTime = MilliSecs () - FramePeriod
MoveMouse GraphicsWidth()/2, GraphicsHeight()/2

Global MyShootRate=200
Global MyShootTimer=TOK_CreateTimer( MyShootRate)

Repeat
	; --------------------------
	; Frame rate limiting
	; --------------------------
	Repeat
		FrameElapsed = MilliSecs () - FrameTime
	Until FrameElapsed

	FrameTicks = FrameElapsed / FramePeriod
	FrameTween# = Float (FrameElapsed Mod FramePeriod) / Float (FramePeriod)

	; -----------------------
	; Update tweening
	; -----------------------
	For FrameLimit = 1 To FrameTicks
		If FrameLimit = FrameTicks Then
			CaptureWorld
		EndIf

		FrameTime = FrameTime + FramePeriod
		
		UpdateGame ()
		UpdateWorld
	Next

	; -------------------
	; Redraw world
	; -------------------
	TOK_RefreshEasyTOK()
	RenderWorld FrameTween
	
	; ---------------------
	; FPS Displaying
	; ---------------------
	Proc_CinemaStrip()
	Color 255,255,255
	Information$="<"+Get_FPS()+" FPS> / "+"<Arrow key to move> / <Mouse left to push> / <Mouse right normal time> / <Mouse meedle reset scene>"
	Text GraphicsWidth()/2,GraphicsHeight()-30,Information$,True,True
	Flip
Until KeyHit (1)

; -----------
; End App
; -----------
TOK_FreePhysic()
FreeFont  AppFont
End

; -----------------------------------
; Tweening update function
; -----------------------------------
Function UpdateGame ()
	Proc_Freelook(Cam,1.05,0.05)
	
	If MouseHit(1) Then
		TOK_ApplyForce("Cylinder",0,0,-7000)
		TOK_DeadRagdoll()
	EndIf

	If MouseHit(3) Then 
		TOK_KillRagdoll()

		For i=1 To 4
			TOK_CreateRagdoll("Doll0"+I,Rnd(-35,35),25,Rnd(-20,20))
		Next

		TOK_DeleteRBEntity("Cylinder")

		Cube_Scale=4
		TOK_CreateRBCylinder("Cylinder",False,0,24,60,90,90,0,Cube_Scale,Cube_Scale*10,20,"",Rnd(255),Rnd(255),Rnd(255),1,0)
		TOK_ApplyMaterial("Cylinder",TOK_TypeRigidBody%,"Ice")
	EndIf
	
	; ---------------------	
	; Update physic
	; ---------------------
	If MouseDown(2)
		TOK_RefreshPhysic(1.7/AppFPS,10)
	Else
		TOK_RefreshPhysic(0.3/AppFPS,10)
	EndIf 
End Function

; -----------------------
; Create the visor
; -----------------------
Function Proc_CreateVisor()
	Visor=LoadSprite("./ETOK_Medias/Target.png")
	PositionEntity Visor,EntityX(Cam),EntityY(Cam),EntityZ(Cam)+2
	ScaleSprite Visor,0.1,0.1
	EntityParent Visor,Cam
	EntityOrder Visor,-1
End Function

; ----------------------------------
; Create the cinema strip
; ----------------------------------
Function Proc_CinemaStrip(Size=60)
	Color 1,1,1
	Rect 0,0,GraphicsWidth(),Size
	Rect 0,GraphicsHeight()-Size,GraphicsWidth(),Size
End Function

; ----------------------
; Camera freelook
; ----------------------
Function Proc_Freelook(CamEntity,Velocity#,Speed#)
	Mouse_X_Speed#=MouseXSpeed()*0.5
	Mouse_Y_Speed#=MouseYSpeed()*0.5

	MoveMouse GraphicsWidth()/2,GraphicsHeight()/2
	Camera_Pitch=Camera_Pitch+Mouse_Y_Speed#
	Camera_Yaw=Camera_Yaw+Mouse_X_Speed#

	RotateEntity CamEntity,Camera_Pitch,-Camera_Yaw,0

	If KeyDown(203) Then
		Camera_VelX=Camera_VelX-Speed#
	ElseIf  KeyDown(205)
		Camera_VelX=Camera_VelX+Speed#
	EndIf
	
	If KeyDown(208) Then
		Camera_VelZ=Camera_VelZ-Speed#
	ElseIf  KeyDown(200)
		Camera_VelZ=Camera_VelZ+Speed#
	EndIf
 
	Camera_VelX=Camera_VelX/velocity#
	Camera_VelZ=Camera_VelZ/velocity#
	MoveEntity CamEntity,Camera_VelX,0,Camera_VelZ

	If KeyDown(200) Then
		MoveEntity CamEntity,0,0,Speed#
	EndIf

	If KeyDown(208) Then
		MoveEntity CamEntity,0,0,-Speed#
	EndIf

	If KeyDown(205) Then
		MoveEntity CamEntity,Speed#,0,0
	EndIf

	If KeyDown(203) Then
		MoveEntity CamEntity,-Speed#,0,0
	EndIf
End Function