;==================================================================
;Project Title: EasyTOK
;Author: Philippe Agnisola
;Email:                    
;Version: Beta 1.0
;Date:              
;Notes: Copyright 2005 Philippe Agnisola (Thanks to Cyril)            
;                   
;                   
;==================================================================

; --------------------------
; TOK Object Type
; --------------------------
Type TOK_ObjectStruct
	Field Name$
	
	Field Body
	Field Mesh
	Field Geom
	
	Field Sound
	Field Channel

	Field BodyType%
	Field LifeTimer

	Field Shininess#
	Field Alpha#

	Field Red%
	Field Green%
	Field Blue%

	Field Pitch#
	Field Yaw#
	Field Roll#
	
	Field Px#
	Field Py#
	Field Pz#

	Field Width#
	Field Height#
	Field Depth#
	Field Mass#

	Field LinearDamping#
	Field AngularDamping#
	Field SleepingParameter#
End Type

; -----------------------
; TOK Object RB
; -----------------------
Type TOK_RigidBody
	Field Name$
	
	Field	Body 
  	Field Geom
End Type

; -----------------------
; TOK Object AB
; -----------------------
Type TOK_AnimatedBody
	Field Name$
	
	Field	Body 
  	Field Geom
End Type

; ------------------------
; TOK Object Join
; ------------------------
Type TOK_ObjectJoint
	Field Name$
	Field ConnectType%
	Field LifeTimer
	
	Field Join
	
	Field BodyA
	Field BodyB

	Field Helper

	Field Pitch#
	Field Yaw#
	Field Roll#
	
	Field Px#
	Field Py#
	Field Pz#
End Type

; --------------------------
; TOK Object bomb
; --------------------------
Type TOK_ObjectBomb
	Field Name$
	
	Field Mesh
	
	Field Explode_Sound
	Field Launch_Sound
		
	Field Detonator
	Field LifeTimer
	Field Status
	Field Activate


	Field Px#
	Field Py#
	Field Pz#

	Field Alpha#
	Field Scale#
	
	Field Radius#
	Field Force#
End Type

; -------------------------------------------
; Type des timers asynchrones
; -------------------------------------------
Type TOK_Timer
	Field Start
	Field TimeOut
End Type

; -----------------------------------------
; Type des matieres tokamak 
; -----------------------------------------
Type TOK_Material
	Field Name$
	Field Index%
	Field Friction#
	Field Restitution#
End Type

; ---------------------------
; Quelques globales
; ---------------------------
Global TOK_InfoBank								;	La banque des collisions
Global TOK_TextureSize						;	Taille de la texture damier
Global TOK_MaterialIndex%=0				;	Permet d'incrementer les ID de matires
Const TOK_JointHelperSize#=1				;	Taille des joint helper

; -----------------------------------------------------
; Flag de collision pour les objets Toka
; -----------------------------------------------------
Const TOK_StaticMeshCollisionID=-1	
Const TOK_AnimBodyCollisionID=2	
Const TOK_RigidBodyCollisionID=3

; -----------------------------------------
; Constantes pour les JOINTS
; -----------------------------------------
Const TOK_BallJoint%=1
Const TOK_HingeJoint%=3
Const TOK_SliderJoint%=4

; ----------------------------------------
; Flag des type d'objets Toka
; ----------------------------------------
Const TOK_TypeRigidBody%=100
Const TOK_TypeAnimBody%=200
Const TOK_TypeStaticMesh%=300

; ---------------------------------------
; EasyTOK objects numbers
; ---------------------------------------
Global TOK_MaxStructObject%
Global TOK_NumStructObject%=0	;	Total object number  (rigid, static, anim)
Global TOK_NumStructJoint%=0	;	Joint number
Global TOK_NumStructBomb%=0	;	Bomb number

; --------------------
; Init toka world
; --------------------
Function TOK_InitWorld(GravityX#=0,Gravity#=-40,GravityZ#=0,NumRB%=500,NumAB%=100)
	TOKSIM_SetRigidBodiesCount NumRB%
	TOKSIM_SetAnimatedBodiesCount NumAB%
	TOKSIM_SetGeometriesCount NumRB%+NumAB%
	TOKSIM_SetOverlappedPairsCount ((NumRB%+NumAB%)*((NumRB%+NumAB%)-1))/2
	
	TOKSIM_CreateSimulator(GravityX#,Gravity#,GravityZ#)

	; -----------------------------------------------------------
	; Creation de la banque pour les collisions
	; -----------------------------------------------------------
	TOK_InfoBank=CreateBank(104*NumRB%)
	TOKSIM_SetCInfoBank TOK_InfoBank

	; -------------------------------------------------
	; Valeur des reponses de collision :
	; -------------------------------------------------
	; 0 = No impulse or feedback
	; 1 = Impulse only
	; 2 = Feedback only
	; 3 = Impulse & Feedback
	; Tout les body avec un CollisionID 0 (ground) & 1 (balls) donneront un collision feedback
	TOKSIM_SetCollisionResponse TOK_RigidBodyCollisionID,TOK_StaticMeshCollisionID,3
	TOKSIM_SetCollisionResponse TOK_RigidBodyCollisionID,TOK_AnimBodyCollisionID,3

	TOKSIM_SetCollisionResponse TOK_AnimBodyCollisionID,TOK_RigidBodyCollisionID,3
	TOKSIM_SetCollisionResponse TOK_AnimBodyCollisionID,TOK_StaticMeshCollisionID,3

	; -----------------------------------------------
	; Definition du nombre max d'objet
	; -----------------------------------------------
	TOK_MaxStructObject%=NumRB%+NumAB%

	; ---------------------------------
	; Definition des matieres
	; ---------------------------------
	TOK_CreateMaterial("Wood",50.4,0.5)	 ; Bois
	TOK_CreateMaterial("Rubber",1.0,.9 )	 ; Caoutchouc
	TOK_CreateMaterial("Ice",0.1,0.3)		 ; Glace/Verre
	TOK_CreateMaterial("Metal",40.4,0.3)	 ; Metal
	TOK_CreateMaterial("Plastic",1.4,0.4)	 ; Plastic
End Function

; ------------------------------------------
; Create a quick tokamak floor
; ------------------------------------------
Function TOK_CreateGround(Name$,Scale#=200,Red=255,Green=255,Blue=255)
	Local Ground_Height#=1
	TOK_CreateABCube(Name$,0,(Ground_Height#/2),0,0,0,0,Scale#,Ground_Height#,Scale#,"",Red,Green,Blue,1)
End Function 

; ------------------------------------------------------------------------
; Create a virtual mesh on terrain to make physique
; ------------------------------------------------------------------------
Function TOK_CreateTerrainPhysique(BlitzTerrain,size#)
	Local_Mesh=LoadMesh("./ETOK_Medias/Terrain_Mesh.b3d")

	ScaleMesh Local_Mesh,Size*0.5,1,Size*0.5
	PositionMesh Local_Mesh,0,0,0
	
	s=GetSurface(Local_Mesh,1)
	
	For k=0 To CountVertices(s)-1
		VertexCoords s,k,VertexX(s,k),TerrainY(BlitzTerrain,VertexX(s,k),0,VertexZ(s,k)),VertexZ(s,k)
	Next

	TOK_CreateTokCollider(Local_Mesh,"","TERRAIN","STATIC",True)
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; --------------------------------------------
; Create an animated body cube
; --------------------------------------------
Function TOK_CreateABCube(Name$,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,Width#=1,Height#=1,Depth#=1,Sound$="",Red=255,Green=255,Blue=255,View%=1)
	G.TOK_ObjectStruct = New TOK_ObjectStruct
	
	G\Name$=Name$
	
	G\Mass#=0
	G\BodyType%=TOK_TypeAnimBody%
	G\Shininess#=0.3
	G\Alpha#=1

	G\Width#=Width#
	G\Height#=Height#
	G\Depth#=Depth#

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%

	G\LifeTimer=-1

	G\Mesh = CreateCube()
	Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
	EntityShininess G\Mesh,G\Shininess#
	EntityTexture G\Mesh,Tok_ObjectTextureTmp
	FreeTexture Tok_ObjectTextureTmp
	
	ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#,1
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#,1
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,1
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	G\Body = TOK_CreateBoxAnimatedBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#*2,G\Height#*2,G\Depth#*2)

	DebugLog "Function : AnimatedBody cube created..."
	Return G\Body
End Function

; -----------------------------------------------
; Create an animated body sphere
; -----------------------------------------------
Function TOK_CreateABSphere(Name$,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,Scale#=1,Sound$="",Red=255,Green=255,Blue=255,View%=1,Segments%=16)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	
	G\BodyType%=TOK_TypeAnimBody%
	G\Shininess#=0.3
	G\Alpha#=1

	G\Width#=Scale#
	G\Height#=Scale#
	G\Depth#=Scale#

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%
	
	G\LifeTimer=-1

	G\Mesh = CreateSphere(Segments%)
	Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
	EntityShininess G\Mesh,G\Shininess#
	EntityTexture G\Mesh,Tok_ObjectTextureTmp
	FreeTexture Tok_ObjectTextureTmp
	
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,1
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#,1
	ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#,1
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

 	G\Body = TOK_CreateSphereAnimatedBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#*2)

	DebugLog "Function : AnimatedBody sphere created..."
	Return G\Body
End Function

; -------------------------------------------------
; Create an animated body cylinder
; -------------------------------------------------
Function TOK_CreateABCylinder(Name$,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,Width#=1,Height#=2,Sound$="",Red=255,Green=255,Blue=255,View%=1,Segments%=16)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	
	G\Mass#=0
	G\BodyType%=TOK_TypeAnimBody%
	G\Shininess#=0.3
	G\Alpha#=1

	G\Width#=Width#
	G\Height#=Height#
	G\Depth#=Width#

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%
	
	G\LifeTimer=-1

	G\Mesh = CreateCylinder(Segments%)
	Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
	EntityShininess G\Mesh,G\Shininess#
	EntityTexture G\Mesh,Tok_ObjectTextureTmp
	FreeTexture Tok_ObjectTextureTmp
	
	ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#,1
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#,1
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,1
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	; -------------------------------------------------------------------
	; Probleme de precision entre Cyl RB et Cyl AB
	; -------------------------------------------------------------------
	G\Body = TOK_CreateCylinderAnimatedBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#*2,G\Height#*2-(G\Width#*2))

	DebugLog "Function : AnimatedBody cylinder created..."
	Return G\Body
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ----------------------------------------------------------------------
; Change a animated body rotation from the name
; ----------------------------------------------------------------------
Function TOK_RotateABEntity(Name$,Pitch#,Yaw#,Roll#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeAnimBody% And Upper$(G\Name$)=Upper$(Name$)
			G\Pitch#=Pitch#
			G\Yaw#=Yaw#
			G\Roll#=Roll#
	
			TOKAB_SetRotation(G\Body,G\Pitch#,G\Yaw#,G\Roll#)
			RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,False
		EndIf
	Next 
End Function

; ----------------------------------------------------------------------
; Change a animated body position from the name
; ----------------------------------------------------------------------
Function TOK_PositionABEntity(Name$,Px#,Py#,Pz#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeAnimBody% And Upper$(G\Name$)=Upper$(Name$)
			G\Px#=Px#
			G\Py#=Py#
			G\Pz#=Pz#
	
			TOKAB_SetPosition(G\Body,G\Px#,G\Py#,G\Pz#)
			PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#,False
		EndIf
	Next 
End Function

; --------------------------------------------------------
; Delete a animated body from the name
; --------------------------------------------------------
Function TOK_DeleteABEntity(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeAnimBody% And Upper$(G\Name$)=Upper$(Name$)
			TOK_DeleteAnimatedBody(G\Body)
			If G\Mesh<>False Then FreeEntity G\Mesh
			Delete G
		EndIf
	Next 
End Function

; -------------------------------------------------------------
; Get the animated body mesh from a name
; ------------------------------------------------------------
Function TOK_GetABMesh(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeAnimBody% And Upper$(name$)=Upper$(G\Name$) Then
			Return G\Mesh
		EndIf 
	Next

	Return False
End Function

; ----------------------------------------------------------------------------
; Return True if the mesh is a AnimatedBody tokamak
; ----------------------------------------------------------------------------
Function TOK_IsAnimatedBody(Mesh)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeAnimBody% And Mesh=G\Mesh Then
			Return True
		EndIf 
	Next

	Return False
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; -----------------------------------
; Create a rigid body cube
; -----------------------------------
Function TOK_CreateRBCube(Name$,Mesh%,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,ScaleX#=1,ScaleY#=1,ScaleZ#=1,Mass#=1,Sound$="",Red%=255,Green%=255,Blue%=255,View%=1,Life%=15)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Shininess#=0.5
	G\Alpha#=1

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%

	G\LinearDamping#=0.0001
	G\AngularDamping#=0.0001
	G\SleepingParameter#=0.01

	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf 

	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	If Mesh%=False Then 
		G\Mesh = CreateCube()
		Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
		
		EntityShininess G\Mesh,G\Shininess#
		EntityTexture G\Mesh,Tok_ObjectTextureTmp
		FreeTexture Tok_ObjectTextureTmp

		G\Width#=ScaleX#
		G\Height#=ScaleY#
		G\Depth#=ScaleZ#
	
		ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#	
	Else
		G\Mesh=CopyEntity(Mesh)
		
		ScaleMesh G\Mesh,ScaleX#,ScaleY#,ScaleZ#
		
		G\Width#=MeshWidth(G\Mesh)
		G\Height#=MeshHeight(G\Mesh)
		G\Depth#=MeshDepth(G\Mesh)
	EndIf 
	
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%
	
	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Mesh%=False Then
 		Double_Size=True
	Else
		Double_Size=False
	EndIf
	
	; ------------------------------------
	; Creation de l'objet TOKA
	; ------------------------------------
	G\Body =TOK_CreateBoxRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,G\Height#,G\Depth#,Double_Size,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter# ,False,G\Mass#)

	Return G\Body
End Function

; --------------------------------------
; Create a rigid body sphere
; --------------------------------------
Function TOK_CreateRBSphere(Name$,Mesh%=False,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,Scale#=1,Mass#=1,Sound$="",Red=255,Green=255,Blue=255,View%=1,Life%=15,Segments%=16)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Shininess#=0.5
	G\Alpha#=1

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%

	G\LinearDamping#=0.0001
	G\AngularDamping#=0.0001
	G\SleepingParameter#=0.01

	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf
	
	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	If Mesh%=False Then 
		G\Mesh = CreateSphere(Segments%)
		Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
		
		EntityShininess G\Mesh,G\Shininess#
		EntityTexture G\Mesh,Tok_ObjectTextureTmp
		FreeTexture Tok_ObjectTextureTmp

		G\Width#=Scale#
		G\Height#=Scale#
		G\Depth#=Scale#
	
		ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#	
	Else
		G\Mesh=CopyEntity(Mesh)
		
		ScaleMesh G\Mesh,Scale#,Scale#,Scale#
		
		G\Width#=MeshWidth(G\Mesh)
		G\Height#=MeshHeight(G\Mesh)
		G\Depth#=MeshDepth(G\Mesh)
	EndIf 
	
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%
	
	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Mesh%=False Then
 		Double_Size=True
	Else
		Double_Size=False
	EndIf

	G\Body = TOK_CreateSphereRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,Double_Size,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter#,False,G\Mass#)
	
	Return G\Body
End Function

; ----------------------------------------
; Create a rigid body cylinder
; ----------------------------------------
Function TOK_CreateRBCylinder(Name$,Mesh%=False,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,Width#=1,Height#=2,Mass#=1,Sound$="",Red=255,Green=255,Blue=255,View%=1,Life%=15,Segments%=16)
	G.TOK_ObjectStruct = New TOK_ObjectStruct
	
	G\Name$=Name$
	
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Shininess#=0.5
	G\Alpha#=1

	G\Pitch#=Pitch#
	G\Yaw#=Yaw#
	G\Roll#=Roll#
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	G\Red%=Red%
	G\Green%=Green%
	G\Blue%=Blue%

	G\LinearDamping#=0.0001
	G\AngularDamping#=0.0001
	G\SleepingParameter#=0.01
	
	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf

	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	If Mesh%=False Then 
		G\Mesh = CreateCylinder(Segments%)
		Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
		
		EntityShininess G\Mesh,G\Shininess#
		EntityTexture G\Mesh,Tok_ObjectTextureTmp
		FreeTexture Tok_ObjectTextureTmp

		G\Width#=Width#
		G\Height#=Height#
		G\Depth#=Width#
	
		ScaleEntity G\Mesh,G\Width#,G\Height#,G\Depth#	
	Else
		G\Mesh=CopyEntity(Mesh)
		
		ScaleMesh G\Mesh,Scale#,Scale#,Scale#
		
		G\Width#=MeshWidth(G\Mesh)
		G\Height#=MeshHeight(G\Mesh)
		G\Depth#=MeshDepth(G\Mesh)
	EndIf 
	
	RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#
	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#
	EntityColor G\Mesh, G\Red%,G\Green%,G\Blue%
	
	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	If Mesh%=False Then
 		Double_Size=True
	Else
		Double_Size=False
	EndIf
	
 	G\Body = TOK_CreateCylinderRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,G\Height#,Double_Size,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter#,False,G\Mass# )

	DebugLog "Function : RigidBody cylinder created..."
	Return G\Body
End Function

; --------------------------------------------------------------
; Change a rigid body rotation from the name
; --------------------------------------------------------------
Function TOK_RotateRBEntity(Name$,Pitch#,Yaw#,Roll#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody% And Upper$(G\Name$)=Upper$(Name$)
			G\Pitch#=Pitch#
			G\Yaw#=Yaw#
			G\Roll#=Roll#

			TOKRB_SetRotation(G\Body,G\Pitch#,G\Yaw#,G\Roll#)
			RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,False
		EndIf
	Next 
End Function

; ---------------------------------------------------------------
; Change a rigid body position from the name
; ---------------------------------------------------------------
Function TOK_PositionRBEntity(Name$,Px#,Py#,Pz#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody% And Upper$(G\Name$)=Upper$(Name$)
			G\Px#=Px#
			G\Py#=Py#
			G\Pz#=Pz#
	
			TOKRB_SetPosition(G\Body,G\Px#,G\Py#,G\Pz#)
			If G\Mesh<>False Then PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#,False
		EndIf
	Next 
End Function

; -------------------------------------------------
; Delete a rigid body from the name
; -------------------------------------------------
Function TOK_DeleteRBEntity(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody% And Upper$(G\Name$)=Upper$(Name$)
			TOK_DeleteRigidBody(G\Body)
			If G\Mesh<>False Then FreeEntity G\Mesh
			Delete G
		EndIf
	Next 
End Function

; ----------------------------------------------------
; Get the rigidbody body from a name
; ----------------------------------------------------
Function TOK_GetRBToka(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
			Return G\Body
		EndIf 
	Next

	Return False
End Function

; -----------------------------------------------------
; Get the rigidbody mesh from a name
; -----------------------------------------------------
Function TOK_GetRBMesh(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
			Return G\Mesh
		EndIf 
	Next

	Return False
End Function

; ----------------------------------------------------------
; Get the rigidbody name from an mesh id
; ----------------------------------------------------------
Function TOK_GetRBName$(Mesh)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Mesh=G\Mesh Then
			Return G\Name$
		EndIf 
	Next

	Return ""
End Function

; -----------------------------------------------
; Get the rigidbody X from a name
; -----------------------------------------------
Function TOK_GetRBX#(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
			Return G\Px#
		EndIf 
	Next

	Return False
End Function

; -----------------------------------------------
; Get the rigidbody X from a name
; -----------------------------------------------
Function TOK_GetRBY#(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
			Return G\Py#
		EndIf 
	Next

	Return False
End Function

; -----------------------------------------------
; Get the rigidbody X from a name
; -----------------------------------------------
Function TOK_GetRBZ#(Name$)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
			Return G\Pz#
		EndIf 
	Next

	Return False
End Function

;------------------------------------------------------
; Get the rigid body pitch from a name
;------------------------------------------------------
Function TOK_GetRBPitch#(Name$,Global_Value=False)
 For G.TOK_ObjectStruct = Each TOK_ObjectStruct
  If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
   Return EntityPitch#(G\Mesh,Global_Value)
  EndIf 
 Next
 
 Return False
End Function

;----------------------------------------------------
; Get the rigid body yaw from a name
;----------------------------------------------------
Function TOK_GetRBYaw#(Name$,Global_Value=False)
 For G.TOK_ObjectStruct = Each TOK_ObjectStruct
  If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
   Return EntityYaw#(G\Mesh,Global_Value)
  EndIf 
 Next
 
 Return False
End Function

;---------------------------------------------------
; Get the rigid body roll from a name
;---------------------------------------------------
Function TOK_GetRBRoll#(Name$,Global_Value=False)
 For G.TOK_ObjectStruct = Each TOK_ObjectStruct
  If  G\BodyType%=TOK_TypeRigidBody% And Upper$(Name$)=Upper$(G\Name$) Then
   Return EntityRoll#(G\Mesh,Global_Value)
  EndIf 
 Next
 
 Return False
End Function

; ---------------------------------------------------------------------
; Return True if the mesh is a RigidBody tokamak
; ---------------------------------------------------------------------
Function TOK_IsRigidBody(Mesh)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If  G\BodyType%=TOK_TypeRigidBody% And Mesh=G\Mesh Then
			Return True
		EndIf 
	Next

	Return False
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; --------------------------------
; Set a rigid body mass
; --------------------------------
Function TOK_SetRBMass(Name$,Mass#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody%
			If Name$="" Then
				G\Mass#=Mass#
				TOKRB_SetMass G\Body,G\Mass#
			Else
				If Upper$(Name$)=Upper$(G\Name$) Then
					G\Mass#=Mass#
					TOKRB_SetMass G\Body,G\Mass#
				EndIf 
			EndIf 
		EndIf 
	Next
End Function

; ---------------------------------------------
; Set a rigid body linear damping
; ---------------------------------------------
Function TOK_SetRBLinearDamping(Name$,LDamping#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody%
			If Name$="" Then
				G\LinearDamping#=LDamping#
				TOKRB_SetLinearDamping G\Body,G\LinearDamping#
			Else
				If Upper$(Name$)=Upper$(G\Name$) Then
					G\LinearDamping#=LDamping#
					TOKRB_SetLinearDamping G\Body,G\LinearDamping#
				EndIf 
			EndIf 
		EndIf 
	Next
End Function

; ------------------------------------------------
; Set a rigid body angular damping
; ------------------------------------------------
Function TOK_SetRBAngularDamping(Name$,ADamping#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody%
			If Name$="" Then
				G\AngularDamping#=ADamping#
				TOKRB_SetAngularDamping G\Body,G\AngularDamping#
			Else
				If Upper$(Name$)=Upper$(G\Name$) Then
					G\AngularDamping#=ADamping#
					TOKRB_SetAngularDamping G\Body,G\AngularDamping#
				EndIf 
			EndIf 
		EndIf 
	Next
End Function

; ---------------------------------------------------
; Set a rigid body sleeping parameter
; ---------------------------------------------------
Function TOK_SetRBSleepingParameter(Name$,Sleeping#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody%
			If Name$="" Then
				G\SleepingParameter#=Sleeping#
				TOKRB_SetSleepingParameter G\Body,G\SleepingParameter#
			Else
				If Upper$(Name$)=Upper$(G\Name$) Then
					G\SleepingParameter#=Sleeping#
					TOKRB_SetSleepingParameter G\Body,G\SleepingParameter#
				EndIf 
			EndIf 
		EndIf 
	Next
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ------------------------------------------
; Texture a rigidbody/animated
; ------------------------------------------
Function TOK_EntityTexture(Name$,Texture)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then 
			EntityTexture G\Mesh,Texture
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				EntityTexture G\Mesh,Texture
			EndIf 
		EndIf 
	Next
End Function

; ---------------------------------------------
; Shininess a rigidbody/animated
; ---------------------------------------------
Function TOK_EntityShininess(Name$,Shininess#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then
			G\Shininess#=Shininess#
			EntityShininess G\Mesh,G\Shininess#
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				G\Shininess#=Shininess#
				EntityShininess G\Mesh,G\Shininess#
			EndIf 
		EndIf 
	Next
End Function

; --------------------------------------------------------------
; Setup the entity type of rigidbody/animated
; --------------------------------------------------------------
Function TOK_EntityType(Name$,EntType%)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then 
			EntityType G\Mesh,EntType%
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				EntityType G\Mesh,EntType%
			EndIf 
		EndIf 
	Next
End Function

; ---------------------------------------
; Alpha a rigidbody/animated
; ---------------------------------------
Function TOK_EntityAlpha(Name$,Alpha#)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then
			G\Alpha#=Alpha#
			EntityAlpha G\Mesh,G\Alpha#
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				G\Alpha#=Alpha#
				EntityAlpha G\Mesh,G\Alpha#
			EndIf 
		EndIf 
	Next
End Function

; ---------------------------------------
; Color a rigidbody/animated
; ---------------------------------------
Function TOK_EntityColor(Name$,Red%,Green%,Blue%)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then
			G\Red%=Red% : G\Green%=Green% : G\Blue%=Blue%
			EntityColor G\Mesh,G\Red%,G\Green%,G\Blue%
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				G\Red%=Red% : G\Green%=Green% : G\Blue%=Blue%
				EntityColor G\Mesh,G\Red%,G\Green%,G\Blue%
			EndIf 
		EndIf 
	Next
End Function

; ----------------------------------------------------------------------
; Setup the entity pickmode of rigidbody/animated
; ----------------------------------------------------------------------
Function TOK_EntityPickMode(Name$,PickMode%)
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If Name$="" Then 
			EntityPickMode G\Mesh,PickMode%
		Else
			If Upper$(Name$)=Upper$(G\Name$) Then
				EntityPickMode G\Mesh,PickMode%
			EndIf 
		EndIf 
	Next
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ---------------------------------------------------------
; Create a tokamak structur from a mesh
; ---------------------------------------------------------
Function TOK_CreatePhysicFromMesh(Parent%)
	If Parent%<>0 Then
		DebugLog "Function : Mesh child count : "+CountChildren(Parent%)
		
 		For a=1 To CountChildren(Parent%)
			Child%=GetChild(Parent%,a)
			TOK_TmpChildName$=Upper$(EntityName$(Child%))
			 
			; --------------------------------------------
			; If the mesh is a RigidBody box
			; --------------------------------------------
			If Instr(TOK_TmpChildName$,"RBOX")<>0
				TOK_TmpMesh=CopyEntity(Child%,0)
				TOK_TmpRBName$=TOK_ExtractString$(TOK_TmpChildName$,"[RBOX=")

				If TOK_StringExist(TOK_TmpChildName$,"MASS")<>0
					TOK_TmpMass#=TOK_ExtractString$(TOK_TmpChildName$,"[MASS=")
				Else
					TOK_TmpMass#=1.0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LDAMP")<>0
					TOK_TmpLDamping#=TOK_ExtractString$(TOK_TmpChildName$,"[LDAMP=")
				Else
					TOK_TmpLDamping#=0.001
				EndIf	

				If TOK_StringExist(TOK_TmpChildName$,"ADAMP")<>0
					TOK_TmpADamping#=TOK_ExtractString$(TOK_TmpChildName$,"[ADAMP=")
				Else
					TOK_TmpADamping#=0.001
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SLEEP")<>0
					TOK_TmpSleeping#=TOK_ExtractString$(TOK_TmpChildName$,"[SLEEP=")
				Else
					TOK_TmpSleeping#=0.06
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LIFE")<>0
					TOK_TmpLife%=TOK_ExtractString$(TOK_TmpChildName$,"[LIFE=")
				Else
					TOK_TmpLife%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"TYPE")<>0
					TOK_TmpType%=TOK_ExtractString$(TOK_TmpChildName$,"[TYPE=")
				Else
					TOK_TmpType%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SOUND")<>0
					TOK_TmpSound$=TOK_ExtractString$(TOK_TmpChildName$,"[SOUND=")
				Else
					TOK_TmpSound$=""
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEX")<>0
					TOK_TmpForceX#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEX=")
				Else
					TOK_TmpForceX#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEY")<>0
					TOK_TmpForceY#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEY=")
				Else
					TOK_TmpForceY#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEZ")<>0
					TOK_TmpForceZ#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEZ=")
				Else
					TOK_TmpForceZ#=0
				EndIf

				DebugLog "Function : RigidBody Box find : "+TOK_TmpRBName$+" / Mass : "+TOK_TmpMass#+" / LDamping : "+TOK_TmpLDamping#+" / ADamping : "+TOK_TmpADamping#+" / Sleeping : "+TOK_TmpSleeping#+" / Life ; "+TOK_TmpLife%+" sec"
				TOK_CreateRBCubeFromChild(TOK_TmpRBName$,TOK_TmpMesh,TOK_TmpSound$,1,TOK_TmpMass#,TOK_TmpLDamping#,TOK_TmpADamping#,TOK_TmpSleeping#,TOK_TmpLife%)

				TOK_EntityType(TOK_TmpRBName$,TOK_TmpType%)
				TOK_ApplyForce(TOK_TmpRBName$,TOK_TmpForceX#,TOK_TmpForceY#,TOK_TmpForceZ#)

			; -------------------------------------------------
			; If the mesh is a RigidBody sphere
			; -------------------------------------------------
			Else If Instr(TOK_TmpChildName$,"RSPHERE")<>0
				TOK_TmpMesh=CopyEntity(Child%,0)
				TOK_TmpRBName$=TOK_ExtractString$(TOK_TmpChildName$,"[RSPHERE=")

				If TOK_StringExist(TOK_TmpChildName$,"MASS")<>0
					TOK_TmpMass#=TOK_ExtractString$(TOK_TmpChildName$,"[MASS=")
				Else
					TOK_TmpMass#=1.0
				EndIf
	
				If TOK_StringExist(TOK_TmpChildName$,"LDAMP")<>0
					TOK_TmpLDamping#=TOK_ExtractString$(TOK_TmpChildName$,"[LDAMP=")
				Else
					TOK_TmpLDamping#=0.001
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"ADAMP")<>0
					TOK_TmpADamping#=TOK_ExtractString$(TOK_TmpChildName$,"[ADAMP=")
				Else
					TOK_TmpADamping#=0.001
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SLEEP")<>0
					TOK_TmpSleeping#=TOK_ExtractString$(TOK_TmpChildName$,"[SLEEP=")
				Else
					TOK_TmpSleeping#=0.01
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LIFE")<>0
					TOK_TmpLife%=TOK_ExtractString$(TOK_TmpChildName$,"[LIFE=")
				Else
					TOK_TmpLife%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"TYPE")<>0
					TOK_TmpType%=TOK_ExtractString$(TOK_TmpChildName$,"[TYPE=")
				Else
					TOK_TmpType%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SOUND")<>0
					TOK_TmpSound$=TOK_ExtractString$(TOK_TmpChildName$,"[SOUND=")
				Else
					TOK_TmpSound$=""
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEX")<>0
					TOK_TmpForceX#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEX=")
				Else
					TOK_TmpForceX#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEY")<>0
					TOK_TmpForceY#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEY=")
				Else
					TOK_TmpForceY#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEZ")<>0
					TOK_TmpForceZ#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEZ=")
				Else
					TOK_TmpForceZ#=0
				EndIf

				DebugLog "Function : RigidBody Sphere find : "+TOK_TmpRBName$+" / Mass : "+TOK_TmpMass#+" / LDamping : "+TOK_TmpLDamping#+" / ADamping : "+TOK_TmpADamping#+" / Sleeping : "+TOK_TmpSleeping#+" / Life ; "+TOK_TmpLife%+" sec"
				TOK_CreateRBSphereFromChild(TOK_TmpRBName$,TOK_TmpMesh,TOK_TmpSound$,1,TOK_TmpMass#,TOK_TmpLDamping#,TOK_TmpADamping#,TOK_TmpSleeping#,TOK_TmpLife%)

				TOK_EntityType(TOK_TmpRBName$,TOK_TmpType%)
				TOK_ApplyForce(TOK_TmpRBName$,TOK_TmpForceX#,TOK_TmpForceY#,TOK_TmpForceZ#)

			; --------------------------------------------------
			; If the mesh is a RigidBody cylinder
			; --------------------------------------------------
			Else If Instr(TOK_TmpChildName$,"RCYLINDER")<>0
				TOK_TmpMesh=CopyEntity(Child%,0)
				TOK_TmpRBName$=TOK_ExtractString$(TOK_TmpChildName$,"[RCYLINDER=")

				If TOK_StringExist(TOK_TmpChildName$,"MASS")<>0
					TOK_TmpMass#=TOK_ExtractString$(TOK_TmpChildName$,"[MASS=")
				Else
					TOK_TmpMass#=1.0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LDAMP")<>0
					TOK_TmpLDamping#=TOK_ExtractString$(TOK_TmpChildName$,"[LDAMP=")
				Else
					TOK_TmpLDamping#=0.001
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"ADAMP")<>0
					TOK_TmpADamping#=TOK_ExtractString$(TOK_TmpChildName$,"[ADAMP=")
				Else
					TOK_TmpADamping#=0.001
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SLEEP")<>0
					TOK_TmpSleeping#=TOK_ExtractString$(TOK_TmpChildName$,"[SLEEP=")
				Else
					TOK_TmpSleeping#=0.01
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LIFE")<>0
					TOK_TmpLife%=TOK_ExtractString$(TOK_TmpChildName$,"[LIFE=")
				Else
					TOK_TmpLife%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"TYPE")<>0
					TOK_TmpType%=TOK_ExtractString$(TOK_TmpChildName$,"[TYPE=")
				Else
					TOK_TmpType%=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"SOUND")<>0
					TOK_TmpSound$=TOK_ExtractString$(TOK_TmpChildName$,"[SOUND=")
				Else
					TOK_TmpSound$=""
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEX")<>0
					TOK_TmpForceX#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEX=")
				Else
					TOK_TmpForceX#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEY")<>0
					TOK_TmpForceY#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEY=")
				Else
					TOK_TmpForceY#=0
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"FORCEZ")<>0
					TOK_TmpForceZ#=TOK_ExtractString$(TOK_TmpChildName$,"[FORCEZ=")
				Else
					TOK_TmpForceZ#=0
				EndIf

				DebugLog "Function : RigidBody Cylinder find : "+TOK_TmpRBName$+" / Mass : "+TOK_TmpMass#+" / LDamping : "+TOK_TmpLDamping#+" / ADamping : "+TOK_TmpADamping#+" / Sleeping : "+TOK_TmpSleeping#+" / Life ; "+TOK_TmpLife%+" sec"
				TOK_CreateRBCylinderFromChild(TOK_TmpRBName$,TOK_TmpMesh,TOK_TmpSound$,1,TOK_TmpMass#,TOK_TmpLDamping#,TOK_TmpADamping#,TOK_TmpSleeping#,TOK_TmpLife%)

				TOK_EntityType(TOK_TmpRBName$,TOK_TmpType%)
				TOK_ApplyForce(TOK_TmpRBName$,TOK_TmpForceX#,TOK_TmpForceY#,TOK_TmpForceZ#)

			; --------------------------------------------------
			; Create an AnimatedBody structure
			; --------------------------------------------------
			Else If Instr(TOK_TmpChildName$,"STATIC")<>0
				TOK_TmpABName$=TOK_ExtractString$(TOK_TmpChildName$,"[STATIC=")

				If TOK_StringExist(TOK_TmpChildName$,"TYPE")<>0
					TOK_TmpType%=TOK_ExtractString$(TOK_TmpChildName$,"[TYPE=")
				Else
					TOK_TmpType%=0
				EndIf
				
				If TOK_StringExist(TOK_TmpChildName$,"SOUND")<>0
					TOK_TmpSound$=TOK_ExtractString$(TOK_TmpChildName$,"[SOUND=")
				Else
					TOK_TmpSound$=""
				EndIf
				
				DebugLog "Function : Structure AnimatedBody find : "+TOK_TmpABName$+" / Collision type : "+TOK_TmpType%
				
				TOK_TmpMesh=CopyEntity(Child%,0)
				TOK_CreateTokCollider(TOK_TmpMesh,TOK_TmpSound$,TOK_TmpABName$,TOK_TmpType%)
			EndIf
		Next

		; ---------------------------
		; Toka Joint creation
		; ---------------------------
 		For a=1 To CountChildren(Parent%)
			Child%=GetChild(Parent%,a)
			TOK_TmpChildName$=Upper$(EntityName$(Child%))

			; -------------------
			; If i find a joint
			; -------------------
			If Instr(TOK_TmpChildName$,"JOINT")<>0
				TOK_TmpJointName$=TOK_ExtractString$(TOK_TmpChildName$,"[JOINT=")

				If TOK_StringExist(TOK_TmpChildName$,"BODYA")<>0
					TOK_TmpBodyA$=TOK_ExtractString$(TOK_TmpChildName$,"[BODYA=")
				Else
					TOK_TmpBodyA$=""
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"BODYB")<>0
					TOK_TmpBodyB$=TOK_ExtractString$(TOK_TmpChildName$,"[BODYB=")
				Else
					TOK_TmpBodyB$=""
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"CONNECT")<>0
					TOK_TmpConnect%=TOK_ExtractString$(TOK_TmpChildName$,"[CONNECT=")
				Else
					TOK_TmpConnect%=1
				EndIf
				
				If TOK_StringExist(TOK_TmpChildName$,"TYPE")<>0
					TOK_TmpType%=TOK_ExtractString$(TOK_TmpChildName$,"[TYPE=")
				Else
					TOK_TmpType%=1
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"DAMPING")<>0
					TOK_TmpDamping#=TOK_ExtractString$(TOK_TmpChildName$,"[DAMPING=")
				Else
					TOK_TmpDamping#=10
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LANGLE")<>0
					TOK_TmpLangle#=TOK_ExtractString$(TOK_TmpChildName$,"[LANGLE=")
				Else
					TOK_TmpLangle#=-90
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"UANGLE")<>0
					TOK_TmpUangle#=TOK_ExtractString$(TOK_TmpChildName$,"[UANGLE=")
				Else
					TOK_TmpUangle#=90
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"VIEW")<>0
					TOK_TmpView%=TOK_ExtractString$(TOK_TmpChildName$,"[VIEW=")
				Else
					TOK_TmpView%=1
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LIMIT")<>0
					TOK_TmpLimit%=TOK_ExtractString$(TOK_TmpChildName$,"[LIMIT=")
				Else
					TOK_TmpLimit%=1
				EndIf

				If TOK_StringExist(TOK_TmpChildName$,"LIFE")<>0
					TOK_TmpLife%=TOK_ExtractString$(TOK_TmpChildName$,"[LIFE=")
				Else
					TOK_TmpLife%=0
				EndIf
				
				DebugLog "LIFE JOINT : "+TOK_TmpLife%
				
				TOK_CreateJoint(TOK_TmpJointName$,TOK_TmpConnect%,TOK_TmpBodyA$,TOK_TmpBodyB$,EntityX(Child%),EntityY(Child%),EntityZ(Child%),EntityPitch(Child%),EntityYaw(Child%),EntityRoll(Child%),TOK_TmpView%,TOK_TmpType%,TOK_TmpDamping#,TOK_TmpLangle#,TOK_TmpUangle#,TOK_TmpLimit%,5,10,TOK_TmpLife%)
			EndIf
		Next

		FreeEntity Parent%
	Else
			DebugLog "Function : Structure object not found !!"
	EndIf
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ---------------------------------
; Create a tokamak joint
; ---------------------------------
Function TOK_CreateJoint(Name$,ConnectType%,BodyA$,BodyB$,Px#=0,Py#=0,Pz#=0,Pitch#=0,Yaw#=0,Roll#=0,HelperView%=1,JoinType%=1,Damping#=30,LAngle#=0,UAngle#=90,Limit%=True,Epsilon#=5,Iteration%=10,Life%=0)
	J.TOK_ObjectJoint=New TOK_ObjectJoint
	
	J\Name$=Name$
	
	J\Px#=Px#
	J\Py#=Py#
	J\Pz#=Pz#

	J\Pitch#=Pitch#
	J\Yaw#=Yaw#
	J\Roll#=Roll#
	
	J\BodyA=TOK_GetRBToka(BodyA$)
	J\BodyB=TOK_GetRBToka(BodyB$)

	J\ConnectType%=ConnectType%

	; -------------------------------------------
	; Creation du timer de fin de vie
	; -------------------------------------------
	If Life%>0 Then 
		J\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		J\LifeTimer=-1
	EndIf
	
	; ----------------------------------------
	; Creation de la sphere d'aide
	; ----------------------------------------
	Local BlueBrush=CreateBrush(0,0,255)
	Local RedBrush=CreateBrush(255,0,0)
	Local GreenBrush=CreateBrush(0,255,0)
	Local GreyBrush=CreateBrush(50,50,50)

	Local Axis01=CreateCylinder(5)
	PaintMesh Axis01,GreenBrush
	
	Local Axis02=CreateCylinder(5)
	PaintMesh Axis02,BlueBrush
	
	Local Axis03=CreateCylinder(5)
	PaintMesh Axis03,RedBrush
	
	J\Helper=CreateSphere(5)
	PaintMesh J\Helper,GreyBrush
	
	ScaleMesh J\Helper,0.9,0.9,0.9
	PositionEntity J\Helper,Px#,Py#,Pz#,1
	
	ScaleMesh Axis01,0.2,TOK_JointHelperSize*1.6,0.2
	PositionEntity Axis01,Px#,Py#,Pz#
	RotateMesh Axis01,0,0,0

	ScaleMesh Axis02,0.2,TOK_JointHelperSize*1.6,0.2
	PositionEntity Axis02,Px#,Py#,Pz#
	RotateMesh Axis02,90,0,0

	ScaleMesh Axis03,0.2,TOK_JointHelperSize*1.6,0.2
	PositionEntity Axis03,Px#,Py#,Pz#
	RotateMesh Axis03,90,90,0
	
	AddMesh Axis01,J\Helper
	AddMesh Axis02,J\Helper
	AddMesh Axis03,J\Helper
	
	FreeEntity Axis01
	FreeEntity Axis02
	FreeEntity Axis03
	
	FreeBrush BlueBrush
	FreeBrush RedBrush
	FreeBrush GreenBrush
	FreeBrush GreyBrush
	
	ScaleEntity J\Helper,TOK_JointHelperSize#,TOK_JointHelperSize#,TOK_JointHelperSize#
	RotateEntity J\Helper,Pitch#,Yaw#,Roll#,1
	EntityShininess J\Helper,0.5
	
	If HelperView%=0 Then
		HideEntity J\Helper
	EndIf

	; --------------------------------
	; Creation du join TOKA
	; --------------------------------
	J\Join=TOKJOINT_Create(J\ConnectType%,J\BodyA,J\BodyB)

	TOKJOINT_SetPositionAndRotationWorld(J\Join,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#)
	TOKJOINT_SetType(J\Join,JoinType%)

	; -----------------------------------------
	; Si le joint n'est pas un slider
	; -----------------------------------------
	If JoinType%<>4
		TOKJOINT_SetLowerLimit J\Join,Pi * (LAngle#) / 180 
		TOKJOINT_SetUpperLimit J\Join,Pi * (UAngle#) / 180 
	Else
		TOKJOINT_SetLowerLimit J\Join,LAngle#
		TOKJOINT_SetUpperLimit J\Join,UAngle#
	EndIf
	
	TOKJOINT_EnableLimit J\Join,Limit%
	TOKJOINT_Enable J\Join,True
	TOKJOINT_SetDampingFactor J\Join,Damping#

	TOKJOINT_SetEpsilon(J\Join,Epsilon#)
	TOKJOINT_SetIterations(J\Join,Iteration%)

	DebugLog "Function : Joint type "+JoinType%+" created..."
	
	Return J\Join
End Function

; --------------------------------------
; Enable or disable the joint
; --------------------------------------
Function TOK_EnableJoint(Name$,State)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Name$="" Then 
			TOKJOINT_Enable(J\Join,State)
		Else
			If Upper$(Name$)=Upper$(J\Name$) Then
				TOKJOINT_Enable(J\Join,State)
			EndIf 
		EndIf 
	Next 
End Function

; ----------------------
; Delete the joint
; ----------------------
Function TOK_DeleteJoint(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Name$="" Then 
			TOKJOINT_Free(J\Join)
			FreeEntity J\Helper
			Delete J
		Else
			If Upper$(Name$)=Upper$(J\Name$) Then
				TOKJOINT_Free(J\Join)
				FreeEntity J\Helper
				Delete J
			EndIf 
		EndIf 
	Next 
End Function

; ---------------------------------------------------------------------------------
; Get the joint state (enable or disable) return true or false
; ---------------------------------------------------------------------------------
Function TOK_GetJointState(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Upper$(Name$)=Upper$(J\Name$) Then
			Return TOKJOINT_IsEnabled(J\Join)
		EndIf 
	Next

	Return False
End Function

; --------------------------------------
; Get the toka ID of the joint
; --------------------------------------
Function TOK_GetJointToka(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Upper$(Name$)=Upper$(J\Name$) Then
			Return J\Join
		EndIf 
	Next

	Return False
End Function

; --------------------------
; Get the joint pitch
; --------------------------
Function TOK_GetJointPitch#(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Upper$(Name$)=Upper$(J\Name$) Then
			Return TOKJOINT_GetFrameBPitch(J\Join)
		EndIf 
	Next

	Return False
End Function

; ------------------------
; Get the joint yaw
; ------------------------
Function TOK_GetJointYaw#(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Upper$(Name$)=Upper$(J\Name$) Then
			Return TOKJOINT_GetFrameBYaw(J\Join)
		EndIf 
	Next

	Return False
End Function

; -----------------------
; Get the joint roll
; -----------------------
Function TOK_GetJointRoll#(Name$)
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		If Upper$(Name$)=Upper$(J\Name$) Then
			Return TOKJOINT_GetFrameBRoll(J\Join)
		EndIf 
	Next

	Return False
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ------------------------------------
; Create Tokamak material
; ------------------------------------
Function TOK_CreateMaterial(Material$,Friction#,Restitution#)
	TOK_MaterialIndex%=TOK_MaterialIndex%+1

	M.TOK_Material=New TOK_Material
	M\Name$=Material$
	M\Index%=TOK_MaterialIndex%
	M\Friction#=Friction#
	M\Restitution#=Restitution#

	TOKSIM_SetMaterial M\Index%,M\Friction#,M\Restitution#
End Function

; -----------------------------------------------------------------------
; Apply a material on Animated Body & Rigid Body
; -----------------------------------------------------------------------
Function TOK_ApplyMaterial(BodyName$,BodyType%,Material$)
	Select  BodyType%

	Case TOK_TypeRigidBody%
		For R.TOK_RigidBody=Each TOK_RigidBody
			If Upper$(R\Name$)=Upper$(BodyName$)
				TOKGEOM_SetMaterialIndex R\Geom,TOK_GetMaterialIndex(Material$)		
			EndIf
		Next

	Case TOK_TypeAnimBody%
		For A.TOK_AnimatedBody=Each TOK_AnimatedBody
			If Upper$(A\Name$)=Upper$(BodyName$)
				TOKGEOM_SetMaterialIndex A\Geom,TOK_GetMaterialIndex(Material$)
			EndIf				
		Next 
	End Select
End Function

; -------------------------------------------------------
; Get material index from material name
; -------------------------------------------------------
Function TOK_GetMaterialIndex(Material$)
	For M.TOK_Material=Each TOK_Material
		If Upper$(M\Name$)=Upper$(Material$)
			Return M\Index%
		EndIf
	Next 
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; -------------------------------
; Update physic object
; -------------------------------
Function TOK_RefreshPhysic(Speed#=1,Precision#=10)
	TOKSIM_Advance(Speed#,Precision#)
End Function

; -----------------------------------
; Update EasyTOK object
; -----------------------------------
Function TOK_RefreshEasyTOK()
	TOK_NumStructObject%=0
	TOK_NumStructJoint%=0
	TOK_NumStructBomb%=0
	
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		TOK_NumStructObject%=TOK_NumStructObject%+1
	
		If G\BodyType%=TOK_TypeStaticMesh% Or G\BodyType%=TOK_TypeRigidBody%  Then
			; ---------------------------------
			; Moving only rigidbody's
			; ---------------------------------
			If G\BodyType%=TOK_TypeRigidBody% And G\Mesh<>False Then
				G\Px#=TOKRB_GetX(G\Body)
				G\Py#=TOKRB_GetY(G\Body)
				G\Pz#=TOKRB_GetZ(G\Body)

				G\Pitch#=TOKRB_GetPitch(G\Body)
				G\Yaw#=TOKRB_GetYaw(G\Body)
				G\Roll#=TOKRB_GetRoll(G\Body)
				
				PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#
 				RotateEntity G\Mesh,G\Pitch#,G\Yaw#,G\Roll#,False
				EntityAlpha G\Mesh,G\Alpha#
			EndIf

			; ---------------------------------
			; Sound collision update
			; ---------------------------------
			If TOKSIM_GetCollisionCount()>0 And G\Sound<>0 Then
				For C=0 To TOKSIM_GetCollisionCount()-1
					YForce#=PeekFloat(TOK_InfoBank,84+(C*104))
										
					If YForce#>1.0 Then				
						If ChannelPlaying(G\Channel)<>1 Then
							G\Channel=PlaySound(G\Sound)
						EndIf 
					EndIf
				Next
			EndIf
			
			; -------------------------------------------------------------------
			; Si la vie de l'objet est termine on baisse l'alpha
			; -------------------------------------------------------------------
			If G\LifeTimer<>-1 And TOK_TimeOut(G\LifeTimer) Then
				G\Alpha#=G\Alpha#-0.01

				; ----------------
				; If alpha = 0
				; ----------------
				If G\Alpha#<=0
					If G\Mesh<>False Then FreeEntity G\Mesh
					TOK_DeleteRigidBody(G\Body)
					
					TOK_NumStructObject%=TOK_NumStructObject%-1
			
					If G\Sound<>0 Then
						If ChannelPlaying(G\Channel)<>1 Then
							StopChannel(G\Channel)
						EndIf
				
						FreeSound G\Sound
					EndIf

					Delete G
				EndIf 
			EndIf 
		EndIf
	Next

	; ------------------
	; Joint update
	; ------------------
	For K.TOK_ObjectJoint=Each TOK_ObjectJoint
		TOK_NumStructJoint%=TOK_NumStructJoint%+1
		
		PositionEntity K\Helper,TOKJOINT_GetFrameAX(K\Join),TOKJOINT_GetFrameAY(K\Join),TOKJOINT_GetFrameAZ(K\Join)
  		RotateEntity K\Helper,TOKJOINT_GetFrameAPitch(K\Join),TOKJOINT_GetFrameAYaw(K\Join),TOKJOINT_GetFrameARoll(K\Join),False
		PositionEntity K\Helper,TOKJOINT_GetFrameBX(K\Join),TOKJOINT_GetFrameBY(K\Join),TOKJOINT_GetFrameBZ(K\Join) 
		RotateEntity K\Helper,TOKJOINT_GetFrameBPitch(K\Join),TOKJOINT_GetFrameBYaw(K\Join),TOKJOINT_GetFrameBRoll(K\Join),False

		If K\LifeTimer<>-1 And TOK_TimeOut(K\LifeTimer) Then
			If K\Helper<>False Then
				FreeEntity K\Helper
			EndIf
			
			TOKJOINT_Free(K\Join)
			TOK_NumStructJoint%=TOK_NumStructJoint%-1
			Delete K
		EndIf 
	Next

	; -------------------
	; Bomb update
	; -------------------
	For B.TOK_ObjectBomb=Each TOK_ObjectBomb
		TOK_NumStructBomb%=TOK_NumStructBomb%+1
		
		If  (TOK_TimeOut(B\Detonator) And B\Status=True) Or (B\Activate=True And B\Status=True) Then
			EntityColor B\Mesh,255,255,0 : PlaySound B\Explode_Sound
			TOK_ApplyExplode(B\Px#,B\Py#,B\Pz#,B\Radius#,B\Force#)
	
			B\LifeTimer=TOK_CreateTimer(2000) : B\Status=False
		EndIf
	
		If B\LifeTimer<>-1 Then 
			If TOK_TimeOut(B\LifeTimer) Then
				TOK_DeleteBomb(B\Name$)
			Else
				B\Alpha#=B\Alpha#-0.02
				B\Scale#=B\Scale#+1.3
				
				ScaleEntity B\Mesh,B\Scale#,B\Scale#,B\Scale#
				EntityAlpha B\Mesh,B\Alpha#
			EndIf
		EndIf 	
	Next 
End Function

; -------------------------------------
; Free all entity and physic
; -------------------------------------
Function TOK_FreePhysic()
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		; --------------------------------------------
		; On free les entites et les sons
		; --------------------------------------------
		If G\Mesh<>False Then 
			FreeEntity G\Mesh
		EndIf 

		; ----------------------------
		; En fonction du type
		; ----------------------------
		Select G\BodyType%
		Case TOK_TypeRigidBody%
			TOK_DeleteRigidBody(G\Body)
		Case TOK_TypeAnimBody% 
			TOK_DeleteAnimatedBody(G\Body)
		End Select
		
		If G\Sound<>0 Then
			FreeSound G\Sound
		EndIf

		Delete G
	Next

	; --------------------------------
	; Destruction des joints
	; --------------------------------
	For J.TOK_ObjectJoint=Each TOK_ObjectJoint
		TOKJOINT_Free(J\Join)
	Next 

	; -------------------------------------------------
	; On libere la banque des collisions
	; -------------------------------------------------
	FreeBank TOK_InfoBank
	TOKSIM_DestroySimulator()
End Function 

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; -----------------------------------------------
; Create static mesh from a mesh
; -----------------------------------------------
Function TOK_CreateTokCollider(Mesh,Sound$,Name$,CollideType%,Erase=False)
	Local offsetv, offsett,scount, vertices, triangles, surface, tric,cvt
	
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	; ------------------------------------
	; Si un son pour les anims
	; ------------------------------------
	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	G\Name$=Name$
	G\BodyType%=TOK_TypeStaticMesh%

	G\Mesh =Mesh
	G\LifeTimer=-1
	
	scount=CountSurfaces(G\Mesh)

	; -----------------------------------
	; On scanne les surfaces
	; -----------------------------------
	For ind=1 To scount
		surface=GetSurface(G\Mesh,ind)
		ttltris=ttltris+CountTriangles(surface)
		ttlvert=ttlvert+CountVertices(surface)
	Next

	vertices=CreateBank(16*ttlvert)
	triangles=CreateBank(24*ttltris)

	offsetv=0
	offsett=0

	For ind=1 To scount
		surface = GetSurface(G\Mesh,ind)
		ctr=CountTriangles(surface)
		tric=tric+cvt
		cvt=CountVertices(surface)

		; ------------------------------------------------------
		; On rempli la banque avec les vertices
		; ------------------------------------------------------
		For v=0 To cvt-1
			TFormPoint VertexX#(surface,v),VertexY#(surface,v),VertexZ#(surface,v),G\mesh,0
		
			PokeFloat vertices,offsetv,TFormedX()
			PokeFloat vertices,offsetv+4,TFormedY() 
			PokeFloat vertices,offsetv+8,TFormedZ()
			PokeFloat vertices,offsetv+12,0.0
			offsetv=offsetv+16
		Next

		; -------------------------------------------------------
		; On rempli la banque avec les triangles
		; -------------------------------------------------------
		For v=0 To ctr-1
			PokeInt triangles,offsett,tric+TriangleVertex(surface,v,0)
			PokeInt triangles,offsett+4,tric+TriangleVertex(surface,v,1)
			PokeInt triangles,offsett+8,tric+TriangleVertex(surface,v,2)
			PokeInt triangles,offsett+12,2 ; Material ID
			PokeInt triangles,offsett+16,0
			PokeInt triangles,offsett+20,0
			offsett=offsett+24
		Next	
	Next

	; -----------------------------------------------------------------------------------------------------------------------------------------------------------------
	; Create static mesh and next if the CollideType%<>0 then assign an entitytype else assign tokamak collision id
	; -----------------------------------------------------------------------------------------------------------------------------------------------------------------
	TOKSIM_SetStaticMesh vertices,ttlvert,triangles,ttltris

	If CollideType%<>0 Then
		EntityType G\Mesh,CollideType%
	Else 
		TOKRB_SetCollisionID G\Mesh,TOK_StaticMeshCollisionID
	EndIf
	
	; --------------------------------
	; On libere les banques
	; --------------------------------
	FreeBank vertices
	FreeBank triangles

	If Erase=True Then
		FreeEntity G\Mesh
		G\Mesh=False
	EndIf 

	DebugLog "Function : Static Mesh structure created..."
End Function

; ---------------------------------
; Create checker texture
; ---------------------------------
Function TOK_CreateCheckerTexture()
	TOK_TextureSize=256
	
	TOK_TextureTmp = CreateTexture(TOK_TextureSize,TOK_TextureSize)
	SetBuffer TextureBuffer(TOK_TextureTmp)

	Color 255,255,0
	Rect 0,0,TOK_TextureSize/2,TOK_TextureSize/2
	Rect TOK_TextureSize/2,TOK_TextureSize/2,TOK_TextureSize/2,TOK_TextureSize/2

	Color 50,100,200
	Rect TOK_TextureSize/2,0,TOK_TextureSize/2,TOK_TextureSize/2
	Rect 0,TOK_TextureSize/2,TOK_TextureSize/2,TOK_TextureSize/2

	ScaleTexture TOK_TextureTmp,0.5,0.5
	SetBuffer BackBuffer()

	Return TOK_TextureTmp
End Function

; ----------------------------------
; Init asynchronous timer
; ----------------------------------
Function TOK_CreateTimer(TimeOut ) 
	Timer.TOK_Timer = New TOK_Timer
	Timer\Start   = MilliSecs() 
	Timer\TimeOut = Timer\start + TimeOut
	
	Return Handle(Timer)
End Function

Function TOK_TimeOut (TheTimer)
	Timer.TOK_Timer = Object.TOK_Timer( TheTimer )	
	
	If Timer <> Null
		If Timer\TimeOut < MilliSecs()
			Delete Timer : Return True
		Else
			Return False
		EndIf
	Else
		Return True
	EndIf
End Function

; --------------------------------------------
; Get if a string is under another
; --------------------------------------------
Function TOK_StringExist(StringScr$,StringFind$)
	If Instr(Upper$(StringScr$),Upper$(StringFind$))
		Return 1
	Else
		Return 0
	EndIf 
End Function 

; -------------------------------
; Token a string from []
; -------------------------------
Function TOK_ExtractString$(StringScr$,StringFind$)
	Str_FindOffset=Instr (StringScr$,StringFind$)
	
	For i=Str_FindOffset+Len(StringFind$) To Len(StringScr$)
		Str_FindChar$=Mid$(StringScr$,i,1)

		If Str_FindChar$="]" Then Exit

		Str_FindString$=Str_FindString$+Str_FindChar$
	Next

	Return Str_FindString$
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ---------------------------------------------
; Create an rigid body type cube
; ---------------------------------------------
Function TOK_CreateBoxRigidBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#,Height#,Depth#,ReScale%,LDamping#,ADamping#,Sleeping#,Connected%,Mass#)
	R.TOK_RigidBody=New TOK_RigidBody

	R\Name$=Name$
	R\Body = TOKRB_Create()

	If ReScale%=0 Then 
  		R\Geom = TOKRB_AddBox(R\Body,Width#,Height#,Depth#)
  		TOKRB_SetBoxInertiaTensor R\Body,Width#,Height#,Depth#,Mass#
	Else
  		R\Geom = TOKRB_AddBox(R\Body,Width#*2,Height#*2,Depth#*2)
  		TOKRB_SetBoxInertiaTensor R\Body,Width#*2,Height#*2,Depth#*2,Mass#
	EndIf

  	TOKGEOM_SetMaterialIndex R\Geom,1
	
  	TOKRB_SetPosition R\Body,Px#,Py#,Pz#
  	TOKRB_SetRotation R\Body,Pitch#,Yaw#,Roll#

  	TOKRB_SetLinearDamping R\Body,LDamping#
  	TOKRB_SetAngularDamping R\Body,ADamping#
	
  	TOKRB_SetMass R\Body,Mass#
	

	TOKRB_SetSleepingParameter R\Body,Sleeping#
	TOKRB_CollideConnected R\Body,Connected%
	TOKRB_SetCollisionID R\Body,TOK_RigidBodyCollisionID
	
	DebugLog "Function : RigidBody box created..."
	Return R\Body
End Function

; -----------------------------------------------
; Create an rigid body type sphere
; -----------------------------------------------
Function TOK_CreateSphereRigidBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#,ReScale%,LDamping#,ADamping#,Sleeping#,Connected%,Mass#)
	R.TOK_RigidBody=New TOK_RigidBody

	R\Name$=Name$
	R\Body = TOKRB_Create()
	
	If ReScale%=0 Then 
  		R\Geom = TOKRB_AddSphere(R\Body,Width#)
  		TOKRB_SetSphereInertiaTensor R\Body,Width#,Mass#
	Else
  		R\Geom = TOKRB_AddSphere(R\Body,Width#*2)
  		TOKRB_SetSphereInertiaTensor R\Body,Width#*2,Mass#
	EndIf

  	TOKGEOM_SetMaterialIndex R\Geom,1
	
  	TOKRB_SetPosition R\Body,Px#,Py#,Pz#
  	TOKRB_SetRotation R\Body,Pitch#,Yaw#,Roll#

  	TOKRB_SetLinearDamping R\Body,LDamping#
  	TOKRB_SetAngularDamping R\Body,ADamping#
	
  	TOKRB_SetMass R\Body,Mass#

	TOKRB_SetSleepingParameter R\Body,Sleeping#
	TOKRB_CollideConnected R\Body,Connected%
	
	TOKRB_SetCollisionID R\Body,TOK_RigidBodyCollisionID

	DebugLog "Function : RigidBody sphere created..."
	Return R\Body
End Function

; -------------------------------------------------
; Create an rigid body type cylinder
; -------------------------------------------------
Function TOK_CreateCylinderRigidBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#,Height#,Rescale%,LDamping#,ADamping#,Sleeping#,Connected%,Mass#)
	R.TOK_RigidBody=New TOK_RigidBody

	R\Name$=Name$
	R\Body = TOKRB_Create()

	If ReScale%=0 Then 
		R\Geom = TOKRB_AddCylinder(R\Body,Width#,Height#-(Width#))
  		TOKRB_SetCylinderInertiaTensor R\Body,Width#,Height#-(Width#),Mass#
	Else
		R\Geom = TOKRB_AddCylinder(R\Body,Width#*2,Height#*2-(Width#*2))
  		TOKRB_SetCylinderInertiaTensor R\Body,Width#*2,Height#*2-(Width#*2),Mass#
	EndIf

  	TOKGEOM_SetMaterialIndex R\Geom,1
	
  	TOKRB_SetPosition R\Body,Px#,Py#,Pz#
  	TOKRB_SetRotation R\Body,Pitch#,Yaw#,Roll#

  	TOKRB_SetLinearDamping R\Body,LDamping#
  	TOKRB_SetAngularDamping R\Body,ADamping#
	
  	TOKRB_SetMass R\Body,Mass#

	TOKRB_SetSleepingParameter R\Body,Sleeping#
	TOKRB_CollideConnected R\Body,Connected%
	
	TOKRB_SetCollisionID R\Body,TOK_RigidBodyCollisionID

	DebugLog "Function : Rigidbody cylinder created..."
	Return R\Body
End Function 

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ----------------------------
;Delete an rigid body
; ----------------------------
Function TOK_DeleteRigidBody(Id)
	Local TOK_BodyCount=0
	
	For R.TOK_RigidBody=Each TOK_RigidBody
		TOK_BodyCount=TOK_BodyCount+1
		
		If R\Body=Id Then
			TOKRB_RemoveGeometry(R\Body,R\Geom)
			TOKRB_Free R\Body
			Delete R
			
			DebugLog "Function : RigidBody n"+TOK_BodyCount+" and geometry deleted..."
			Exit 
		EndIf 
	Next 
End Function

; -----------------------------------
;Delete an animated body
; -----------------------------------
Function TOK_DeleteAnimatedBody(Id)
	Local TOK_BodyCount=0
	
	For A.TOK_AnimatedBody=Each TOK_AnimatedBody
		TOK_BodyCount=TOK_BodyCount+1
			
		If A\Body=Id Then
			TOKAB_RemoveGeometry(A\Body,A\Geom)
			TOKAB_Free A\Body
			Delete A

			DebugLog "Function : AnimatedBody n"+TOK_BodyCount+" and geometry deleted..."
			Exit 
		EndIf 
	Next 
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ---------------------------------------------------
; Create an animated body type cube
; ---------------------------------------------------
Function TOK_CreateBoxAnimatedBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#,Height#,Depth#)
	A.TOK_AnimatedBody=New TOK_AnimatedBody

	A\Name$=Name$
	
	A\Body = TOKAB_Create()
	A\Geom = TOKAB_AddBox(A\Body,Width#,Height#,Depth#)
	
  	TOKGEOM_SetMaterialIndex A\Geom,1
  	TOKAB_SetPosition A\Body,Px#,Py#,Pz#
  	TOKAB_SetRotation A\Body,Pitch#,Yaw#,Roll#
	
	TOKAB_SetCollisionID A\Body,TOK_AnimBodyCollisionID
	
	Return A\Body
End Function

; ------------------------------------------------------
; Create an animated body type sphere
; ------------------------------------------------------
Function TOK_CreateSphereAnimatedBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#)
	A.TOK_AnimatedBody=New TOK_AnimatedBody

	A\Name$=Name$
	
	A\Body = TOKAB_Create()
	A\Geom = TOKAB_AddSphere(A\Body,Width#)
	
  	TOKGEOM_SetMaterialIndex A\Geom,1	
  	TOKAB_SetPosition A\Body,Px#,Py#,Pz#
  	TOKAB_SetRotation A\Body,Pitch#,Yaw#,Roll#
	
	TOKAB_SetCollisionID A\Body,TOK_AnimBodyCollisionID
	
	Return A\Body
End Function

; --------------------------------------------------------
; Create an animated body type cylinder
; --------------------------------------------------------
Function TOK_CreateCylinderAnimatedBody(Name$,Px#,Py#,Pz#,Pitch#,Yaw#,Roll#,Width#,Height#)
	A.TOK_AnimatedBody=New TOK_AnimatedBody

	A\Name$=Name$
	
	A\Body = TOKAB_Create()
	A\Geom = TOKAB_AddCylinder(A\Body,Width#*2,Height#*2-(Width#*2))
	
  	TOKGEOM_SetMaterialIndex A\Geom,1	
  	TOKAB_SetPosition A\Body,Px#,Py#,Pz#
  	TOKAB_SetRotation A\Body,Pitch#,Yaw#,Roll#
	
	TOKAB_SetCollisionID A\Body,TOK_AnimBodyCollisionID
	
	Return A\Body
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

 ; ------------------------------------------------------
; Create an RB Cube from a mesh child
; -------------------------------------------------------
Function TOK_CreateRBCubeFromChild(Name$,Mesh,Sound$="",View%=1,Mass#=1,LDamping#=0.001,ADamping#=0.001,Sleeping#=0.01,Life%=0)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	G\Mesh = Mesh
	
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Alpha#=1

	G\Pitch#=EntityPitch(G\Mesh,True)
	G\Yaw#=EntityYaw(G\Mesh,True)
	G\Roll#=EntityRoll(G\Mesh,True)

	G\Width#=MeshWidth(G\Mesh)
	G\Height#=MeshHeight(G\Mesh)
	G\Depth#=MeshDepth(G\Mesh)
	
	G\Px#=EntityX(G\Mesh,True)
	G\Py#=EntityY(G\Mesh,True)
	G\Pz#=EntityZ(G\Mesh,True)

	G\LinearDamping#=LDamping#
	G\AngularDamping#=ADamping#
	G\SleepingParameter#=Sleeping#

	; --------------------------
	; Si l'objet a une vie
	; --------------------------
	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf 

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	; ------------------------------------
	; Si un son pour les anims
	; ------------------------------------
	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	; ------------------------------------
	; Creation de l'objet TOKA
	; ------------------------------------
 	G\Body = TOK_CreateBoxRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,G\Height#,G\Depth#,False,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter#,False,G\Mass#)
End Function

; ----------------------------------------------------------
; Create an RB Sphere from a mesh child
; ----------------------------------------------------------
Function TOK_CreateRBSphereFromChild(Name$,Mesh,Sound$="",View%=1,Mass#=1,LDamping#=0.001,ADamping#=0.001,Sleeping#=0.01,Life%=0)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	G\Mesh = Mesh
	
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Alpha#=1

	G\Pitch#=EntityPitch(G\Mesh,True)
	G\Yaw#=EntityYaw(G\Mesh,True)
	G\Roll#=EntityRoll(G\Mesh,True)

	G\Width#=MeshWidth(G\Mesh)
	G\Height#=MeshHeight(G\Mesh)
	G\Depth#=MeshDepth(G\Mesh)
	
	G\Px#=EntityX(G\Mesh,True)
	G\Py#=EntityY(G\Mesh,True)
	G\Pz#=EntityZ(G\Mesh,True)

	G\LinearDamping#=LDamping#
	G\AngularDamping#=ADamping#
	G\SleepingParameter#=Sleeping#
	
	; --------------------------
	; Si l'objet a une vie
	; --------------------------
	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf 

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	; ------------------------------------
	; Si un son pour les anims
	; ------------------------------------
	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	; ------------------------------------
	; Creation de l'objet TOKA
	; ------------------------------------
 	G\Body = TOK_CreateSphereRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,False,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter#,False,G\Mass#)
End Function

; -----------------------------------------------------------
; Create an RB Cylinder from a mesh child
; -----------------------------------------------------------
Function TOK_CreateRBCylinderFromChild(Name$,Mesh,Sound$="",View%=1,Mass#=1,LDamping#=0.001,ADamping#=0.001,Sleeping#=0.01,Life%=0)
	G.TOK_ObjectStruct = New TOK_ObjectStruct

	G\Name$=Name$
	G\Mesh = Mesh
	
	G\Mass#=Mass#
	G\BodyType%=TOK_TypeRigidBody%
	G\Alpha#=1

	G\Pitch#=EntityPitch(G\Mesh,True)
	G\Yaw#=EntityYaw(G\Mesh,True)
	G\Roll#=EntityRoll(G\Mesh,True)

	G\Width#=MeshWidth(G\Mesh)
	G\Height#=MeshHeight(G\Mesh)
	G\Depth#=MeshDepth(G\Mesh)
	
	G\Px#=EntityX(G\Mesh,True)
	G\Py#=EntityY(G\Mesh,True)
	G\Pz#=EntityZ(G\Mesh,True)

	G\LinearDamping#=LDamping#
	G\AngularDamping#=ADamping#
	G\SleepingParameter#=Sleeping#
	
	; --------------------------
	; Si l'objet a une vie
	; --------------------------
	If Life%>0 Then 
		G\LifeTimer=TOK_CreateTimer(Life%*1000)
	Else
		G\LifeTimer=-1
	EndIf 

	If View%=0 Then
		HideEntity G\Mesh
	EndIf

	; ------------------------------------
	; Si un son pour les anims
	; ------------------------------------
	If Sound$<>"" Then 
		G\Sound=LoadSound(Sound$)
	Else
		G\Sound=0
	EndIf

	; ------------------------------------
	; Creation de l'objet TOKA
	; ------------------------------------
 	G\Body = TOK_CreateCylinderRigidBody(G\Name$,G\Px#,G\Py#,G\Pz#,G\Pitch#,G\Yaw#,G\Roll#,G\Width#,G\Height#,False,G\LinearDamping#,G\AngularDamping#,G\SleepingParameter#,False,G\Mass#)
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

Function TOK_ApplyExplode(WorldX#,WorldY#,WorldZ#,Power#,Radius#)
	Center=CreatePivot()
	PositionEntity Center,WorldX#,WorldY#,WorldZ#
	
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody% And G\Mesh<>False Then
			D#=EntityDistance(Center,G\Mesh)

			If D#<=Radius#
				PointEntity Center,G\Mesh
				TOK_ApplyImpulseWithAngle(TOK_GetRBName$(G\mesh),EntityPitch(Center,True),EntityYaw(Center,True),Power#)
			EndIf
		EndIf 
	Next

	FreeEntity Center
End Function

; ----------------------------------------------
; Apply an impulse from an angle
; ----------------------------------------------
Function TOK_ApplyImpulseWithAngle(Name$,Pitch#,Yaw#,ForcePower#)
	TOK_TmpPivot=CreatePivot()
	RotateEntity TOK_TmpPivot,Pitch,Yaw,0
	MoveEntity TOK_TmpPivot,0,0,ForcePower#
	
	For G.TOK_ObjectStruct = Each TOK_ObjectStruct
		If G\BodyType%=TOK_TypeRigidBody%
			If Name$="" Then 
				TOKRB_ApplyImpulse G\Body,EntityX(TOK_TmpPivot),EntityY(TOK_TmpPivot),EntityZ(TOK_TmpPivot)
			Else
				If Upper$(Name$)=Upper$(G\Name$) Then
					TOKRB_ApplyImpulse G\Body,EntityX(TOK_TmpPivot),EntityY(TOK_TmpPivot),EntityZ(TOK_TmpPivot)
				EndIf 
			EndIf 
		EndIf 
	Next

	FreeEntity TOK_TmpPivot
End Function

; -----------------------------------------------
; Apply an impulse from a position
; -----------------------------------------------
Function TOK_ApplyImpulse(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_ApplyImpulse G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_ApplyImpulse G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; -----------------------------------------------------------
; Apply an impulse2 from a position/vector
; -----------------------------------------------------------
Function TOK_ApplyImpulse2(Name$,Fx#,Fy#,Fz#,Px#,Py#,Pz#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_ApplyImpulse2 G\Body,Fx#,Fy#,Fz#,Px#,Py#,Pz#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_ApplyImpulse2 G\Body,Fx#,Fy#,Fz#,Px#,Py#,Pz#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; ---------------------------------------
; Apply a twist to a direction
; ---------------------------------------
Function TOK_ApplyTwist(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_ApplyTwist G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_ApplyTwist G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; ---------------------------------------
; Apply a force to a direction
; ---------------------------------------
Function TOK_ApplyForce(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_SetForce G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_SetForce G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; -------------------------------------------------------------
; Apply a force to a direction from a position
; -------------------------------------------------------------
Function TOK_ApplyForce2(Name$,X#,Y#,Z#,XPos#,YPos#,ZPos#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_SetForce2 G\Body,X#,Y#,Z#,XPos#,YPos#,ZPos#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_SetForce2 G\Body,X#,Y#,Z#,XPos#,YPos#,ZPos#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; -----------------------------------------
; Apply a torque to a direction
; -----------------------------------------
Function TOK_ApplyTorque(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					 TOKRB_SetTorque G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						 TOKRB_SetTorque G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; ------------------------------------------
; Apply a velocity to a direction
; ------------------------------------------
Function TOK_ApplyVelocity(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					 TOKRB_SetVelocity G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						 TOKRB_SetVelocity G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; ---------------------------------------------------------
; Apply angular momentum to a direction
; ---------------------------------------------------------
Function TOK_ApplyAngularMomentum(Name$,X#,Y#,Z#)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					 TOKRB_SetAngularMomentum G\Body,X#,Y#,Z#
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						 TOKRB_SetAngularMomentum G\Body,X#,Y#,Z#
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; -------------------------------------------------------
; Enable/display gravity to a Rigid Body
; -------------------------------------------------------
Function TOK_ApplyGravity(Name$,Gravity%)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_GravityEnable G\Body, Gravity%
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_GravityEnable G\Body, Gravity%
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; ------------------------------------------------------
; Enable/disable active to a Rigid Body
; ------------------------------------------------------
Function TOK_ApplyActive(Name$,Active%)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_Active G\Body, Active%
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_Active G\Body, Active%
					EndIf 
				EndIf 
			EndIf 
		Next
End Function


; -----------------------------------------
; is gravity enabled for this Rigid Body?
; -----------------------------------------
Function TOK_IsGravityEnabled%(Name$)
		For G.TOK_ObjectStruct = Each TOK_ObjectStruct
			If G\BodyType%=TOK_TypeRigidBody%
				If Name$="" Then 
					TOKRB_IsGravityEnabled G\Body%
				Else
					If Upper$(Name$)=Upper$(G\Name$) Then
						TOKRB_IsGravityEnabled G\Body%
					EndIf 
				EndIf 
			EndIf 
		Next
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

; ------------------------------
; Create a static bomb
; ------------------------------
Function TOK_CreateBomb(Name$,Px#,Py#,Pz#,Radius#,Force#,Second=30,Launch_Sound$="./ETOK_Medias/Explosion.mp3",Explode_Sound$="./ETOK_Medias/Explosion.mp3")
	G.TOK_ObjectBomb=New TOK_ObjectBomb

	G\Name$=Name$

	G\Alpha#=1.0
	G\Scale#=2.0
	
	G\Mesh=CreateSphere(7)
	Tok_ObjectTextureTmp=TOK_CreateCheckerTexture()
	EntityTexture G\Mesh,Tok_ObjectTextureTmp
	FreeTexture Tok_ObjectTextureTmp

	EntityShininess G\Mesh,1
	EntityAlpha G\Mesh,G\Alpha#
	ScaleEntity G\Mesh,G\Scale#,G\Scale#,G\Scale#
	
	G\Explode_Sound=LoadSound(Explode_Sound$)
	G\Launch_Sound=LoadSound(Launch_Sound$)
	PlaySound G\Launch_Sound

	G\Detonator=TOK_CreateTimer(Second*1000)
	G\LifeTimer=-1
	G\Status=True
	G\Activate=False
	
	G\Px#=Px#
	G\Py#=Py#
	G\Pz#=Pz#

	PositionEntity G\Mesh,G\Px#,G\Py#,G\Pz#
	
	G\Radius#=Radius#
	G\Force#=Force#

	Return Handle(G)
End Function

Function TOK_DeleteBomb(Name$)
	For G.TOK_ObjectBomb=Each TOK_ObjectBomb
		If Upper$(Name$)=Upper$(G\Name$) Then
			FreeSound G\Launch_Sound
			FreeSound G\Explode_Sound
			FreeEntity G\Mesh
			Delete G
		EndIf 
	Next 
End Function

Function TOK_ActivateAllBomb()
	For G.TOK_ObjectBomb=Each TOK_ObjectBomb
		G\Activate=True
	Next 
End Function

; -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
